using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Globalization;
using System.Resources;
using System.Web.Mail;
using System.Reflection;
using System.Threading;


namespace ChessApplication
{	
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class ChessWindow : System.Windows.Forms.Form
	{
		static string FileToOpen;
        private delegate ChessMove GenerateMoveDelegate();
		private System.Windows.Forms.StatusBar statusBar1;
		private ChessApplication.ImageGrid chessGrid2;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.ComponentModel.IContainer components;		
		//private bool hasComputerPlayer;
		//private ChessPieceColor computerSide;
		private ChessPlayerType BlackPlayer;
		private ChessPlayerType WhitePlayer;
		private ChessPieceColor MyColor;
		private string BlackEmailAddress;
		private string WhiteEmailAddress;
		private string GameName;
		private int ListenPort;
		private bool hintGeneration;		
		private bool isGenerating;
		private ChessBoard chessboard;
		private Stack historyBoard;
		private Stack historyMove;
		private bool ChangedSinceSaved;
		private EmilChessService ChessService;
		private bool ShowProgress;
		private ChessPieceColor ColorAtBottom;
		private Image BlackBishopImage;
		private Image BlackKingImage;
		private Image BlackKnightImage;
		private Image BlackPawnImage;
		private Image BlackQueenImage;
		private Image BlackRookImage;
		private Image WhiteBishopImage;
		private Image WhiteKingImage;
		private Image WhiteKnightImage;
		private Image WhitePawnImage;
		private Image WhiteQueenImage;
		private Image WhiteRookImage;
		private Color WindowBack;
		private Color WindowFore;
		private Color StatusBarFontColor;
		private Color StatusBarProgressBarColor;
		private Brush BlackBrush;
		private Brush WhiteBrush;

		private XmlDocument ConfigXmlDoc;
		private double BishopValue;
		private double KnightValue;
		private double PawnValue;
		private double QueenValue;
		private double RookValue;
		private string SkinPath;
		private int ServicePort;
		private string EmailAddress;
		private string SMTPServer;
		private string LanguageCode;

		private string ProgramDir;
		private string DataDir;
		private string HelpFilePath;

		//private GenerateMoveDelegate GenerateMoveCallback;
		private ChessEngine engine;
		private IAsyncResult GenerateMoveAsyncResult;
		private ResourceManager rm;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.ListBox historyListBox;
		private System.Windows.Forms.Label nextPlayerLabel;
		private System.Windows.Forms.Timer pollTimer;
		private System.Windows.Forms.MenuItem FileMI;
		private System.Windows.Forms.MenuItem HelpMI;
		private System.Windows.Forms.MenuItem HelpAboutMI;
		private System.Windows.Forms.MenuItem FileExitMI;
		private System.Windows.Forms.MenuItem CommandMI;
		private System.Windows.Forms.MenuItem FileSaveGameMI;
		private System.Windows.Forms.MenuItem FileLoadGameMI;
		private System.Windows.Forms.MenuItem CommandBackward1MoveMI;
		private System.Windows.Forms.MenuItem CommandBaclward2MovesMI;
		private System.Windows.Forms.MenuItem CommandSuggestMoveMI;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.SaveFileDialog saveFileDialog1;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.Label BlackPlayerColorLabel;
		private System.Windows.Forms.Label WhitePlayerColorLabel;
		private System.Windows.Forms.MenuItem Claim50MovesRuleMI;
		private System.Windows.Forms.MenuItem ClaimDraw3StateRepeatRuleMI;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Timers.Timer highlightTimer;
		private System.Windows.Forms.MenuItem ShowPossibleMovesForPieceMI;
		private System.Windows.Forms.Label MoveHistoryLabel;
		private System.Windows.Forms.Label PlayerToMoveLabel;
		private System.Windows.Forms.Label BlackPlayerLabel;
		private System.Windows.Forms.Label WhitePlayerLabel;
		private System.Windows.Forms.MenuItem FileOptionsMI;
		private System.Windows.Forms.MenuItem FileNewGameMI;
		
		ChessAppNewGameDlg ngd;

		private string BlackInCheckStr;
		private string WhiteInCheckStr;
		private string GameEndedInDrawStr;
		private string BlackWonStr;
		private string WhiteWonStr;
		private string BlackStr;
		private string WhiteStr;
		private string ComputerStr;
		private string HumanStr;
		private string HumanIPNetworkClientStr;
		private string HumanIPNetworkServerStr;
		private string HumanEmailStr;
		private string[] ChessPlayerTypeStrings;
		private string MeStr;
		private string NoPieceSelectedStr;
		private string ChooseBlackInsteadStr;
		private string ChooseWhiteInsteadStr;
		private string WouldPutBlackInCheckStr;
		private string WouldPutWhiteInCheckStr;
		private string IllegalMoveStr;
		private string IllegalDrawClaimStr;
		private string NrMovesStr;
		private string SureOfQuitingStr;
		private string SureOfQuiting_titleStr;
		private string Illegal_move_not_my_turnStr;
		private string Sending_mail_failed_becauseStr;
		private string Invalid_smtp_serverStr;
		private string Change_smtp_server_withStr;
		private string Error_loading_skinStr;
        private string Error_loading_configuration_fileStr;
		private string Error_saving_configuration_fileStr;
		private string Error_loading_gameStr;
		private string Error_saving_gameStr;
		private string Lost_connectionStr;

		private System.Windows.Forms.MenuItem FileNewGameServerMI;
		private System.Windows.Forms.MenuItem FileNewGameClientMI;
		private System.Windows.Forms.Timer NetworkTimer;
		private System.Windows.Forms.TextBox SendMsgTB;
		private System.Windows.Forms.TextBox MessagesTB;
		private string UnableToStepBackStr;
		private System.Windows.Forms.MenuItem ViewMI;
		private System.Windows.Forms.MenuItem ViewBlackAtBottomMI;
		private System.Windows.Forms.MenuItem ViewWhiteAtBottomMI;
		private System.Windows.Forms.MenuItem FileNewGameViaEmailMI;
		private System.Windows.Forms.MenuItem menuItem3;
		private ChessApplication.ImageGrid Captured1IG;
		private ChessApplication.ImageGrid Captured2IG;
		private System.Windows.Forms.MenuItem HelpContentsMI;
		private System.Windows.Forms.MenuItem HelpIndexMI;
		private System.Windows.Forms.MenuItem menuItem5;

		private StatusBarPanel StatusBarPanel1;
        /// <summary> Main windows form.</summary>
		public ChessWindow()
		{
			//
			// Required for Windows Form Designer support
			//
	
			InitializeComponent();
			StatusBarFontColor = Color.Black;
			Assembly a=Assembly.GetExecutingAssembly();
			ProgramDir = a.Location;
			int rem = 1+a.GetName().Name.Length+4;
			// Remove "\" sign Assembly name and ".exe" from the path
			ProgramDir = ProgramDir.Remove(ProgramDir.Length-rem,rem);

			//MessageBox.Show(ProgramDir);
			//ProgramDir = Environment.CurrentDirectory;

			DataDir = ProgramDir + @"\Data";

			ShowProgress = false;
			ColorAtBottom = ChessPieceColor.White;

			BishopValue = 3;
			KnightValue = 3;
			PawnValue = 1;
			QueenValue = 9;
			RookValue = 5;

			BlackBishopImage = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackBishop.bmp");
			BlackKingImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackKing.bmp");
			BlackKnightImage = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackKnight.bmp");
			BlackPawnImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackPawn.bmp");
			BlackQueenImage  = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackQueen.bmp");
			BlackRookImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\BlackRook.bmp");
			WhiteBishopImage = new Bitmap(ProgramDir + "\\Skins\\Original\\WhiteBishop.bmp");
			WhiteKingImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\WhiteKing.bmp");
			WhiteKnightImage = new Bitmap(ProgramDir + "\\Skins\\Original\\WhiteKnight.bmp");
			WhitePawnImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\WhitePawn.bmp");
			WhiteQueenImage  = new Bitmap(ProgramDir + "\\Skins\\Original\\WhiteQueen.bmp");
			WhiteRookImage   = new Bitmap(ProgramDir + "\\Skins\\Original\\WhiteRook.bmp");
			Color backColor = Color.Blue; 
			// Make backColor transparent for myBitmap.

			((Bitmap)BlackBishopImage).MakeTransparent(backColor);
			((Bitmap)BlackKingImage).MakeTransparent(backColor);
			((Bitmap)BlackKnightImage).MakeTransparent(backColor);
			((Bitmap)BlackPawnImage).MakeTransparent(backColor);
			((Bitmap)BlackQueenImage).MakeTransparent(backColor);
			((Bitmap)BlackRookImage).MakeTransparent(backColor);
			((Bitmap)WhiteBishopImage).MakeTransparent(backColor);
			((Bitmap)WhiteKingImage).MakeTransparent(backColor);
			((Bitmap)WhiteKnightImage).MakeTransparent(backColor);
			((Bitmap)WhitePawnImage).MakeTransparent(backColor);
			((Bitmap)WhiteQueenImage).MakeTransparent(backColor);
			((Bitmap)WhiteRookImage).MakeTransparent(backColor);
			WindowBack = this.BackColor;
			WindowFore = this.ForeColor;
			BlackBrush = new SolidBrush(Color.Red);
			WhiteBrush = new SolidBrush(Color.White);

			if (ColorAtBottom == ChessPieceColor.Black)
			{
				Captured2IG.DarkBackBrush = BlackBrush;
				Captured2IG.LightBackBrush = BlackBrush; 
				Captured1IG.DarkBackBrush = WhiteBrush;
				Captured1IG.LightBackBrush = WhiteBrush;
			}
			else
			{
				Captured1IG.DarkBackBrush = BlackBrush;
				Captured1IG.LightBackBrush = BlackBrush; 
				Captured2IG.DarkBackBrush = WhiteBrush;
				Captured2IG.LightBackBrush = WhiteBrush;
			}

			LoadConfig();
			if (LanguageCode != null && LanguageCode != "")
				Thread.CurrentThread.CurrentUICulture = new CultureInfo(LanguageCode);
			rm = new ResourceManager("ChessApplication.Strings",  
				this.GetType().Assembly);
			this.Text = rm.GetString("ChessWindow");
			FileMI.Text = rm.GetString("FileMI");
			FileNewGameMI.Text = rm.GetString("FileNewGameMI");
			FileNewGameClientMI.Text = rm.GetString("FileNewGameClientMI");
			FileNewGameServerMI.Text = rm.GetString("FileNewGameServerMI");
			FileNewGameViaEmailMI.Text = rm.GetString("FileNewGameViaEmailMI");
			FileSaveGameMI.Text = rm.GetString("FileSaveGameMI");
			FileLoadGameMI.Text = rm.GetString("FileLoadGameMI");
			FileOptionsMI.Text = rm.GetString("FileOptionsMI");
			FileExitMI.Text = rm.GetString("FileExitMI");
			ViewMI.Text = rm.GetString("ViewMI");
			ViewBlackAtBottomMI.Text = rm.GetString("ViewBlackAtBottomMI");
			ViewWhiteAtBottomMI.Text = rm.GetString("ViewWhiteAtBottomMI");
			CommandMI.Text = rm.GetString("CommandMI");
			CommandBackward1MoveMI.Text = rm.GetString("CommandBackward1MoveMI");
			CommandBaclward2MovesMI.Text = rm.GetString("CommandBaclward2MovesMI");
			Claim50MovesRuleMI.Text = rm.GetString("Claim50MovesRuleMI");
			ClaimDraw3StateRepeatRuleMI.Text = rm.GetString("ClaimDraw3StateRepeatRuleMI");
			CommandSuggestMoveMI.Text = rm.GetString("CommandSuggestMoveMI");
			ShowPossibleMovesForPieceMI.Text = rm.GetString("ShowPossibleMovesForPieceMI");
			HelpMI.Text = rm.GetString("HelpMI");
			HelpContentsMI.Text = rm.GetString("HelpContentsMI");
			HelpIndexMI.Text = rm.GetString("HelpIndexMI");
			HelpAboutMI.Text = rm.GetString("HelpAboutMI");
			MoveHistoryLabel.Text = rm.GetString("MoveHistoryLabel");
			
			PlayerToMoveLabel.Text = rm.GetString("PlayerToMoveLabel");
			BlackPlayerColorLabel.Text = rm.GetString("BlackPlayerColorLabel");
			WhitePlayerColorLabel.Text = rm.GetString("WhitePlayerColorLabel");
			BlackPlayerLabel.Text = rm.GetString("BlackPlayerLabel");
			WhitePlayerLabel.Text = rm.GetString("WhitePlayerLabel");
			nextPlayerLabel.Text = rm.GetString("White");

			BlackInCheckStr = rm.GetString("BlackInCheck");
			WhiteInCheckStr = rm.GetString("WhiteInCheck");
			GameEndedInDrawStr = rm.GetString("GameEndedInDraw");
			BlackWonStr = rm.GetString("BlackWon");
			WhiteWonStr = rm.GetString("WhiteWon");
			BlackStr = rm.GetString("Black");
			WhiteStr = rm.GetString("White");
			ComputerStr = rm.GetString("Computer");
		    HumanStr = rm.GetString("Human");
			HumanIPNetworkClientStr = rm.GetString("HumanIPNetworkClient");
			HumanIPNetworkServerStr = rm.GetString("HumanIPNetworkServer");
			HumanEmailStr = rm.GetString("HumanEmail");
			ChessPlayerTypeStrings = new string[5];
			ChessPlayerTypeStrings[(int)ChessPlayerType.Human] = HumanStr;
			ChessPlayerTypeStrings[(int)ChessPlayerType.Computer] = ComputerStr;
			ChessPlayerTypeStrings[(int)ChessPlayerType.HumanIPNetworkClient] = HumanIPNetworkClientStr;
			ChessPlayerTypeStrings[(int)ChessPlayerType.HumanIPNetworkServer] = HumanIPNetworkServerStr;
			ChessPlayerTypeStrings[(int)ChessPlayerType.HumanEmail] = HumanEmailStr;
			MeStr = rm.GetString("Me");
			NoPieceSelectedStr = rm.GetString("NoPieceSelected");
			ChooseBlackInsteadStr = rm.GetString("ChooseBlackInstead");
			ChooseWhiteInsteadStr = rm.GetString("ChooseWhiteInstead");
			WouldPutBlackInCheckStr = rm.GetString("WouldPutBlackInCheck");
			WouldPutWhiteInCheckStr = rm.GetString("WouldPutWhiteInCheck");
			IllegalMoveStr = rm.GetString("IllegalMove");
			IllegalDrawClaimStr = rm.GetString("IllegalDrawClaim");
			NrMovesStr = rm.GetString("NrMoves");
			SureOfQuitingStr = rm.GetString("SureOfQuiting");
			SureOfQuiting_titleStr = rm.GetString("SureOfQuiting_title");
			UnableToStepBackStr = rm.GetString("UnableToStepBack");
			Illegal_move_not_my_turnStr = rm.GetString("Illegal_move_not_my_turn");
			Sending_mail_failed_becauseStr = rm.GetString("Sending_mail_failed_because");
			Invalid_smtp_serverStr = rm.GetString("Invalid_smtp_server");
			Change_smtp_server_withStr = rm.GetString("Change_smtp_server_with");
		    Error_loading_skinStr = rm.GetString("Error_loading_skin");
            Error_loading_configuration_fileStr = rm.GetString("Error_loading_configuration_file");
			Error_saving_configuration_fileStr = rm.GetString("Error_saving_configuration_file");
		    Error_loading_gameStr = rm.GetString("Error_loading_game");
		    Error_saving_gameStr = rm.GetString("Error_saving_game");
			Lost_connectionStr = rm.GetString("Lost_connection");
			
		    HelpFilePath = DataDir + @"\" + rm.GetString("HelpFileName");
				//nextPlayerLabel = new System.Windows.Forms.Label();
			
			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			StatusBarPanel1 = new StatusBarPanel();
			StatusBarPanel1.AutoSize = StatusBarPanelAutoSize.Spring;
			StatusBarPanel1.Style = StatusBarPanelStyle.OwnerDraw;
			statusBar1.Panels.Add(StatusBarPanel1);
			statusBar1.ShowPanels = true;

			//LoadConfig();

			chessboard       = new ChessBoard(BishopValue, KnightValue,
				PawnValue, QueenValue, RookValue);
            historyBoard = new Stack();
            historyMove = new Stack();
		    //computerSide = ChessPieceColor.Black;
		    //hasComputerPlayer = true;
			hintGeneration = false;
			isGenerating = false;
			ChangedSinceSaved = true;
			//GenerateMoveCallback = new GenerateMoveDelegate(chessboard.GenerateOrdinaryMove);
            //GenerateMoveCallback = new GenerateMoveDelegate(chessboard.DecideMove);
			//engine = new SimpleEngine(chessboard,4);
			engine = new AlphaBetaEngine(chessboard,historyBoard,4);
			//engine = new SearchNBestEngine(chessboard);

			//statusBar1.
			//LoadSkin("Skins\\Standard.xml");
			BlackPlayer = ChessPlayerType.Computer;
		    WhitePlayer = ChessPlayerType.Human;
			MyColor = ChessPieceColor.White;
			EmilChessService.SetChessWindow(this);

			if (ChessWindow.FileToOpen != null && ChessWindow.FileToOpen != "")
			{
				LoadGame(FileToOpen);
			}
			ngd = new ChessAppNewGameDlg(rm,HelpFilePath);
		}
		/// <summary>
		/// The type of the Black player.
		/// </summary>
		public ChessPlayerType BlackPlayerType
		{
			get { return BlackPlayer; }
			set { BlackPlayer = value; }
		}
		/// <summary>
		/// The type of the white player.
		/// </summary>
		public ChessPlayerType WhitePlayerType
		{
			get { return WhitePlayer; }
			set { WhitePlayer = value; }
		}

		/// <summary>
		/// What my color is My Color.
		/// </summary>
		public ChessPieceColor MYColor
		{
			get { return MyColor; }
			set { MyColor = value; }
		}

		/// <summary>
		/// The port that the Chess game server will be listening to.
		/// </summary>
		public int Port
		{
			get { return ListenPort; }
			set { ListenPort = value; }
		}

		private ChessPosition GridPoint2ChessPosition(Point gridPoint)
		{
			if (ColorAtBottom == ChessPieceColor.Black)
				return new ChessPosition(8-gridPoint.X,gridPoint.Y+1);
			else
				return new ChessPosition(gridPoint.X+1,8-gridPoint.Y);
		}

		private Point ChessPositionGridPoint(ChessPosition position)
		{
			if (ColorAtBottom == ChessPieceColor.Black)
				return new Point(8-position.Column,position.Row-1);
			else
				return new Point(position.Column-1,8-position.Row);
		}

		private void DisplayPiece(ChessPosition position)
		{
			if (chessboard.IsSqareOccupied(position))
			{
				if (chessboard.GetPiece(position).IsColorBlack())
				{
					if (chessboard.GetPiece(position) is Bishop)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackBishopImage);
					else if (chessboard.GetPiece(position) is King)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackKingImage);
					else if (chessboard.GetPiece(position) is Knight)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackKnightImage);
					else if (chessboard.GetPiece(position) is Pawn)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackPawnImage);
					else if (chessboard.GetPiece(position) is Queen)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackQueenImage);
					else if (chessboard.GetPiece(position) is Rook)
						chessGrid2.SetImage(ChessPositionGridPoint(position),BlackRookImage);
				}
				else
				{
					if (chessboard.GetPiece(position) is Bishop)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhiteBishopImage);
					else if (chessboard.GetPiece(position) is King)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhiteKingImage);
					else if (chessboard.GetPiece(position) is Knight)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhiteKnightImage);
					else if (chessboard.GetPiece(position) is Pawn)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhitePawnImage);
					else if (chessboard.GetPiece(position) is Queen)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhiteQueenImage);
					else if (chessboard.GetPiece(position) is Rook)
						chessGrid2.SetImage(ChessPositionGridPoint(position),WhiteRookImage);
				}
			}
			else
				chessGrid2.SetImage(ChessPositionGridPoint(position),null);

		}

		private void DisplayChessBoard()
		{
			if (chessboard.ColorToMove == ChessPieceColor.Black)
                nextPlayerLabel.Text = BlackStr;
			else
				nextPlayerLabel.Text = WhiteStr;
			for(int row=1; row<=8; row++)
				for(int column=1; column<=8; column++)
				{
					DisplayPiece(new ChessPosition(column,row));
				}
			DisplayCapturedPieces();
		}

		private void DisplayCapturedPieces()
		{
			int BlackBishops;
			int BlackKnights;
            int BlackPawns;
			int BlackQueens;
			int BlackRooks;
            int WhiteBishops;
			int WhiteKnights;
            int WhitePawns;
			int WhiteQueens;
			int WhiteRooks;
			ImageGrid BlackIG,WhiteIG;
			int i;
			int imageIndex;
			Captured1IG.ClearImages();
			Captured2IG.ClearImages();
			if (ColorAtBottom == ChessPieceColor.Black)
			{
				BlackIG = Captured1IG;
				WhiteIG = Captured2IG;
			}
			else
			{
				BlackIG = Captured2IG;
				WhiteIG = Captured1IG;
			}
			
			chessboard.CapturedPieces(out BlackBishops, out BlackKnights,
			out BlackPawns, out BlackQueens, out BlackRooks,
			out WhiteBishops, out WhiteKnights,
			out WhitePawns, out WhiteQueens, out WhiteRooks);
			imageIndex = 0;
			for(i=0; i<BlackQueens; ++i)
				BlackIG.SetImage(new Point(imageIndex++,0),BlackQueenImage);
			for(i=0; i<BlackRooks; ++i)
				BlackIG.SetImage(new Point(imageIndex++,0),BlackRookImage);
			for(i=0; i<BlackBishops; ++i)
				BlackIG.SetImage(new Point(imageIndex++,0),BlackBishopImage);
			for(i=0; i<BlackKnights; ++i)
				BlackIG.SetImage(new Point(imageIndex++,0),BlackKnightImage);
			for(i=0; i<BlackPawns; ++i)
				BlackIG.SetImage(new Point(imageIndex++,0),BlackPawnImage);
			imageIndex = 0;
			for(i=0; i<WhiteQueens; ++i)
				WhiteIG.SetImage(new Point(imageIndex++,0),WhiteQueenImage);
			for(i=0; i<WhiteRooks; ++i)
				WhiteIG.SetImage(new Point(imageIndex++,0),WhiteRookImage);
			for(i=0; i<WhiteBishops; ++i)
				WhiteIG.SetImage(new Point(imageIndex++,0),WhiteBishopImage);
			for(i=0; i<WhiteKnights; ++i)
				WhiteIG.SetImage(new Point(imageIndex++,0),WhiteKnightImage);
			for(i=0; i<WhitePawns; ++i)
				WhiteIG.SetImage(new Point(imageIndex++,0),WhitePawnImage);
			Captured1IG.Refresh();
			Captured2IG.Refresh();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ChessWindow));
            this.statusBar1 = new System.Windows.Forms.StatusBar();
            this.mainMenu1 = new System.Windows.Forms.MainMenu(this.components);
            this.FileMI = new System.Windows.Forms.MenuItem();
            this.FileNewGameMI = new System.Windows.Forms.MenuItem();
            this.FileNewGameClientMI = new System.Windows.Forms.MenuItem();
            this.FileNewGameServerMI = new System.Windows.Forms.MenuItem();
            this.FileNewGameViaEmailMI = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.FileSaveGameMI = new System.Windows.Forms.MenuItem();
            this.FileLoadGameMI = new System.Windows.Forms.MenuItem();
            this.FileOptionsMI = new System.Windows.Forms.MenuItem();
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.FileExitMI = new System.Windows.Forms.MenuItem();
            this.ViewMI = new System.Windows.Forms.MenuItem();
            this.ViewBlackAtBottomMI = new System.Windows.Forms.MenuItem();
            this.ViewWhiteAtBottomMI = new System.Windows.Forms.MenuItem();
            this.CommandMI = new System.Windows.Forms.MenuItem();
            this.CommandBackward1MoveMI = new System.Windows.Forms.MenuItem();
            this.CommandBaclward2MovesMI = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.Claim50MovesRuleMI = new System.Windows.Forms.MenuItem();
            this.ClaimDraw3StateRepeatRuleMI = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.CommandSuggestMoveMI = new System.Windows.Forms.MenuItem();
            this.ShowPossibleMovesForPieceMI = new System.Windows.Forms.MenuItem();
            this.HelpMI = new System.Windows.Forms.MenuItem();
            this.HelpContentsMI = new System.Windows.Forms.MenuItem();
            this.HelpIndexMI = new System.Windows.Forms.MenuItem();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.HelpAboutMI = new System.Windows.Forms.MenuItem();
            this.panel1 = new System.Windows.Forms.Panel();
            this.MessagesTB = new System.Windows.Forms.TextBox();
            this.SendMsgTB = new System.Windows.Forms.TextBox();
            this.nextPlayerLabel = new System.Windows.Forms.Label();
            this.PlayerToMoveLabel = new System.Windows.Forms.Label();
            this.BlackPlayerColorLabel = new System.Windows.Forms.Label();
            this.WhitePlayerColorLabel = new System.Windows.Forms.Label();
            this.BlackPlayerLabel = new System.Windows.Forms.Label();
            this.WhitePlayerLabel = new System.Windows.Forms.Label();
            this.historyListBox = new System.Windows.Forms.ListBox();
            this.MoveHistoryLabel = new System.Windows.Forms.Label();
            this.pollTimer = new System.Windows.Forms.Timer(this.components);
            this.saveFileDialog1 = new System.Windows.Forms.SaveFileDialog();
            this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
            this.highlightTimer = new System.Timers.Timer();
            this.NetworkTimer = new System.Windows.Forms.Timer(this.components);
            this.Captured2IG = new ChessApplication.ImageGrid();
            this.Captured1IG = new ChessApplication.ImageGrid();
            this.chessGrid2 = new ChessApplication.ImageGrid();
            this.panel1.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.highlightTimer)).BeginInit();
            this.SuspendLayout();
            // 
            // statusBar1
            // 
            this.statusBar1.Location = new System.Drawing.Point(0, 304);
            this.statusBar1.Name = "statusBar1";
            this.statusBar1.Size = new System.Drawing.Size(504, 16);
            this.statusBar1.TabIndex = 0;
            this.statusBar1.TextChanged += new System.EventHandler(this.statusBar1_TextChanged);
            this.statusBar1.PanelClick += new System.Windows.Forms.StatusBarPanelClickEventHandler(this.statusBar1_PanelClick);
            this.statusBar1.DrawItem += new System.Windows.Forms.StatusBarDrawItemEventHandler(this.statusBar1_DrawItem);
            // 
            // mainMenu1
            // 
            this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.FileMI,
            this.ViewMI,
            this.CommandMI,
            this.HelpMI});
            // 
            // FileMI
            // 
            this.FileMI.Index = 0;
            this.FileMI.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.FileNewGameMI,
            this.FileNewGameClientMI,
            this.FileNewGameServerMI,
            this.FileNewGameViaEmailMI,
            this.menuItem3,
            this.FileSaveGameMI,
            this.FileLoadGameMI,
            this.FileOptionsMI,
            this.menuItem1,
            this.FileExitMI});
            this.FileMI.Text = "File";
            // 
            // FileNewGameMI
            // 
            this.FileNewGameMI.Index = 0;
            this.FileNewGameMI.Text = "New Game";
            this.FileNewGameMI.Click += new System.EventHandler(this.FileNewGameMI_Click);
            // 
            // FileNewGameClientMI
            // 
            this.FileNewGameClientMI.Index = 1;
            this.FileNewGameClientMI.Text = "New Game Client";
            this.FileNewGameClientMI.Click += new System.EventHandler(this.FileNewGameClientMI_Click);
            // 
            // FileNewGameServerMI
            // 
            this.FileNewGameServerMI.Index = 2;
            this.FileNewGameServerMI.Text = "New Game Server";
            this.FileNewGameServerMI.Click += new System.EventHandler(this.NewNetworkGameMI_Click);
            // 
            // FileNewGameViaEmailMI
            // 
            this.FileNewGameViaEmailMI.Index = 3;
            this.FileNewGameViaEmailMI.Text = "New Game via E-mail";
            this.FileNewGameViaEmailMI.Click += new System.EventHandler(this.FileNewGameViaEmailMI_Click);
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 4;
            this.menuItem3.Text = "-";
            // 
            // FileSaveGameMI
            // 
            this.FileSaveGameMI.Index = 5;
            this.FileSaveGameMI.Text = "Save Game";
            this.FileSaveGameMI.Click += new System.EventHandler(this.FileSaveGameMI_Click);
            // 
            // FileLoadGameMI
            // 
            this.FileLoadGameMI.Index = 6;
            this.FileLoadGameMI.Text = "Load Game";
            this.FileLoadGameMI.Click += new System.EventHandler(this.FileLoadGameMI_Click);
            // 
            // FileOptionsMI
            // 
            this.FileOptionsMI.Index = 7;
            this.FileOptionsMI.Text = "Options";
            this.FileOptionsMI.Click += new System.EventHandler(this.FileOptionsMI_Click);
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 8;
            this.menuItem1.Text = "-";
            // 
            // FileExitMI
            // 
            this.FileExitMI.Index = 9;
            this.FileExitMI.Text = "Exit";
            this.FileExitMI.Click += new System.EventHandler(this.menuItem4_Click);
            // 
            // ViewMI
            // 
            this.ViewMI.Index = 1;
            this.ViewMI.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.ViewBlackAtBottomMI,
            this.ViewWhiteAtBottomMI});
            this.ViewMI.Text = "View";
            // 
            // ViewBlackAtBottomMI
            // 
            this.ViewBlackAtBottomMI.Index = 0;
            this.ViewBlackAtBottomMI.Text = "Black at bottom";
            this.ViewBlackAtBottomMI.Click += new System.EventHandler(this.ViewBlackAtBottomMI_Click);
            // 
            // ViewWhiteAtBottomMI
            // 
            this.ViewWhiteAtBottomMI.Checked = true;
            this.ViewWhiteAtBottomMI.Index = 1;
            this.ViewWhiteAtBottomMI.Text = "White at bottom";
            this.ViewWhiteAtBottomMI.Click += new System.EventHandler(this.ViewWhiteAtBottomMI_Click);
            // 
            // CommandMI
            // 
            this.CommandMI.Index = 2;
            this.CommandMI.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.CommandBackward1MoveMI,
            this.CommandBaclward2MovesMI,
            this.menuItem2,
            this.Claim50MovesRuleMI,
            this.ClaimDraw3StateRepeatRuleMI,
            this.menuItem4,
            this.CommandSuggestMoveMI,
            this.ShowPossibleMovesForPieceMI});
            this.CommandMI.Text = "Command";
            // 
            // CommandBackward1MoveMI
            // 
            this.CommandBackward1MoveMI.Enabled = false;
            this.CommandBackward1MoveMI.Index = 0;
            this.CommandBackward1MoveMI.Text = "Backward 1 move";
            this.CommandBackward1MoveMI.Click += new System.EventHandler(this.CommandBackward1MoveMI_Click);
            // 
            // CommandBaclward2MovesMI
            // 
            this.CommandBaclward2MovesMI.Enabled = false;
            this.CommandBaclward2MovesMI.Index = 1;
            this.CommandBaclward2MovesMI.Text = "Backward 2 moves";
            this.CommandBaclward2MovesMI.Click += new System.EventHandler(this.CommandBaclward2MovesMI_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 2;
            this.menuItem2.Text = "-";
            // 
            // Claim50MovesRuleMI
            // 
            this.Claim50MovesRuleMI.Index = 3;
            this.Claim50MovesRuleMI.Text = "Claim draw because 50 moves rule";
            this.Claim50MovesRuleMI.Click += new System.EventHandler(this.Claim50MovesRuleMI_Click);
            // 
            // ClaimDraw3StateRepeatRuleMI
            // 
            this.ClaimDraw3StateRepeatRuleMI.Index = 4;
            this.ClaimDraw3StateRepeatRuleMI.Text = "Claim draw because state repeated 3 times";
            this.ClaimDraw3StateRepeatRuleMI.Click += new System.EventHandler(this.ClaimDraw3StateRepeatRuleMI_Click);
            // 
            // menuItem4
            // 
            this.menuItem4.Index = 5;
            this.menuItem4.Text = "-";
            // 
            // CommandSuggestMoveMI
            // 
            this.CommandSuggestMoveMI.Index = 6;
            this.CommandSuggestMoveMI.Text = "Suggest move";
            this.CommandSuggestMoveMI.Click += new System.EventHandler(this.CommandSuggestMoveMI_Click);
            // 
            // ShowPossibleMovesForPieceMI
            // 
            this.ShowPossibleMovesForPieceMI.Index = 7;
            this.ShowPossibleMovesForPieceMI.Text = "Show Possible Moves For Piece";
            this.ShowPossibleMovesForPieceMI.Click += new System.EventHandler(this.ShowPossibleMovesForPieceMI_Click);
            // 
            // HelpMI
            // 
            this.HelpMI.Index = 3;
            this.HelpMI.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.HelpContentsMI,
            this.HelpIndexMI,
            this.menuItem5,
            this.HelpAboutMI});
            this.HelpMI.Text = "Help";
            // 
            // HelpContentsMI
            // 
            this.HelpContentsMI.Index = 0;
            this.HelpContentsMI.Text = "Contents";
            this.HelpContentsMI.Click += new System.EventHandler(this.HelpContentsMI_Click);
            // 
            // HelpIndexMI
            // 
            this.HelpIndexMI.Index = 1;
            this.HelpIndexMI.Text = "Index";
            this.HelpIndexMI.Click += new System.EventHandler(this.HelpIndexMI_Click);
            // 
            // menuItem5
            // 
            this.menuItem5.Index = 2;
            this.menuItem5.Text = "-";
            // 
            // HelpAboutMI
            // 
            this.HelpAboutMI.Index = 3;
            this.HelpAboutMI.Text = "About";
            this.HelpAboutMI.Click += new System.EventHandler(this.HelpAboutMI_Click);
            // 
            // panel1
            // 
            this.panel1.BackColor = System.Drawing.SystemColors.Control;
            this.panel1.Controls.Add(this.MessagesTB);
            this.panel1.Controls.Add(this.SendMsgTB);
            this.panel1.Controls.Add(this.nextPlayerLabel);
            this.panel1.Controls.Add(this.PlayerToMoveLabel);
            this.panel1.Controls.Add(this.BlackPlayerColorLabel);
            this.panel1.Controls.Add(this.WhitePlayerColorLabel);
            this.panel1.Controls.Add(this.BlackPlayerLabel);
            this.panel1.Controls.Add(this.WhitePlayerLabel);
            this.panel1.Controls.Add(this.historyListBox);
            this.panel1.Controls.Add(this.MoveHistoryLabel);
            this.panel1.Location = new System.Drawing.Point(344, 0);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(160, 240);
            this.panel1.TabIndex = 2;
            // 
            // MessagesTB
            // 
            this.MessagesTB.Location = new System.Drawing.Point(0, 136);
            this.MessagesTB.Multiline = true;
            this.MessagesTB.Name = "MessagesTB";
            this.MessagesTB.Size = new System.Drawing.Size(152, 72);
            this.MessagesTB.TabIndex = 12;
            this.MessagesTB.Visible = false;
            // 
            // SendMsgTB
            // 
            this.SendMsgTB.Location = new System.Drawing.Point(0, 216);
            this.SendMsgTB.Name = "SendMsgTB";
            this.SendMsgTB.Size = new System.Drawing.Size(136, 20);
            this.SendMsgTB.TabIndex = 11;
            this.SendMsgTB.Visible = false;
            this.SendMsgTB.TextChanged += new System.EventHandler(this.SendMsgTB_TextChanged);
            this.SendMsgTB.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.SendMsgTB_KeyPress);
            // 
            // nextPlayerLabel
            // 
            this.nextPlayerLabel.Location = new System.Drawing.Point(96, 40);
            this.nextPlayerLabel.Name = "nextPlayerLabel";
            this.nextPlayerLabel.Size = new System.Drawing.Size(56, 16);
            this.nextPlayerLabel.TabIndex = 6;
            this.nextPlayerLabel.Text = "White";
            // 
            // PlayerToMoveLabel
            // 
            this.PlayerToMoveLabel.Location = new System.Drawing.Point(8, 40);
            this.PlayerToMoveLabel.Name = "PlayerToMoveLabel";
            this.PlayerToMoveLabel.Size = new System.Drawing.Size(88, 16);
            this.PlayerToMoveLabel.TabIndex = 5;
            this.PlayerToMoveLabel.Text = "Player to move:";
            // 
            // BlackPlayerColorLabel
            // 
            this.BlackPlayerColorLabel.Location = new System.Drawing.Point(96, 24);
            this.BlackPlayerColorLabel.Name = "BlackPlayerColorLabel";
            this.BlackPlayerColorLabel.Size = new System.Drawing.Size(64, 16);
            this.BlackPlayerColorLabel.TabIndex = 4;
            this.BlackPlayerColorLabel.Text = "Computer";
            // 
            // WhitePlayerColorLabel
            // 
            this.WhitePlayerColorLabel.Location = new System.Drawing.Point(96, 8);
            this.WhitePlayerColorLabel.Name = "WhitePlayerColorLabel";
            this.WhitePlayerColorLabel.Size = new System.Drawing.Size(64, 16);
            this.WhitePlayerColorLabel.TabIndex = 3;
            this.WhitePlayerColorLabel.Text = "Human";
            // 
            // BlackPlayerLabel
            // 
            this.BlackPlayerLabel.Location = new System.Drawing.Point(8, 24);
            this.BlackPlayerLabel.Name = "BlackPlayerLabel";
            this.BlackPlayerLabel.Size = new System.Drawing.Size(88, 16);
            this.BlackPlayerLabel.TabIndex = 2;
            this.BlackPlayerLabel.Text = "Black player:";
            // 
            // WhitePlayerLabel
            // 
            this.WhitePlayerLabel.Location = new System.Drawing.Point(8, 8);
            this.WhitePlayerLabel.Name = "WhitePlayerLabel";
            this.WhitePlayerLabel.Size = new System.Drawing.Size(88, 16);
            this.WhitePlayerLabel.TabIndex = 1;
            this.WhitePlayerLabel.Text = "White player:";
            this.WhitePlayerLabel.Click += new System.EventHandler(this.label1_Click);
            // 
            // historyListBox
            // 
            this.historyListBox.ForeColor = System.Drawing.SystemColors.WindowText;
            this.historyListBox.Location = new System.Drawing.Point(0, 80);
            this.historyListBox.Name = "historyListBox";
            this.historyListBox.Size = new System.Drawing.Size(144, 108);
            this.historyListBox.TabIndex = 0;
            // 
            // MoveHistoryLabel
            // 
            this.MoveHistoryLabel.Location = new System.Drawing.Point(8, 64);
            this.MoveHistoryLabel.Name = "MoveHistoryLabel";
            this.MoveHistoryLabel.Size = new System.Drawing.Size(136, 23);
            this.MoveHistoryLabel.TabIndex = 7;
            this.MoveHistoryLabel.Text = "Move history:";
            // 
            // pollTimer
            // 
            this.pollTimer.Tick += new System.EventHandler(this.pollTimer_Tick);
            // 
            // saveFileDialog1
            // 
            this.saveFileDialog1.DefaultExt = "cgs";
            this.saveFileDialog1.FileName = "ChessGame1";
            this.saveFileDialog1.Filter = "\"Chess Game Saves (*.cgs)|*.cgs";
            this.saveFileDialog1.Title = "Save Chess game";
            // 
            // openFileDialog1
            // 
            this.openFileDialog1.DefaultExt = "cgs";
            this.openFileDialog1.Filter = "Chess Game Saves (*.cgs)|*.cgs";
            this.openFileDialog1.Title = "Open Chess Game Saves";
            // 
            // highlightTimer
            // 
            this.highlightTimer.Interval = 3000;
            this.highlightTimer.SynchronizingObject = this;
            this.highlightTimer.Elapsed += new System.Timers.ElapsedEventHandler(this.highlightTimer_Elapsed);
            // 
            // NetworkTimer
            // 
            this.NetworkTimer.Tick += new System.EventHandler(this.NetworkTimer_Tick);
            // 
            // Captured2IG
            // 
            this.Captured2IG.GridSize = new System.Drawing.Size(17, 1);
            this.Captured2IG.HasGridLabels = false;
            this.Captured2IG.HighlightingColor = System.Drawing.Color.Green;
            this.Captured2IG.Location = new System.Drawing.Point(0, 232);
            this.Captured2IG.Name = "Captured2IG";
            this.Captured2IG.SelectedImage = new System.Drawing.Point(-1, -1);
            this.Captured2IG.SelectionColor = System.Drawing.Color.Purple;
            this.Captured2IG.SelectionLineWidth = 0.1F;
            this.Captured2IG.Size = new System.Drawing.Size(344, 32);
            this.Captured2IG.TabIndex = 4;
            // 
            // Captured1IG
            // 
            this.Captured1IG.GridSize = new System.Drawing.Size(17, 1);
            this.Captured1IG.HasGridLabels = false;
            this.Captured1IG.HighlightingColor = System.Drawing.Color.Green;
            this.Captured1IG.Location = new System.Drawing.Point(0, 0);
            this.Captured1IG.Name = "Captured1IG";
            this.Captured1IG.SelectedImage = new System.Drawing.Point(-1, -1);
            this.Captured1IG.SelectionColor = System.Drawing.Color.Purple;
            this.Captured1IG.SelectionLineWidth = 0.1F;
            this.Captured1IG.Size = new System.Drawing.Size(344, 32);
            this.Captured1IG.TabIndex = 3;
            // 
            // chessGrid2
            // 
            this.chessGrid2.GridSize = new System.Drawing.Size(8, 8);
            this.chessGrid2.HasGridLabels = true;
            this.chessGrid2.HighlightingColor = System.Drawing.Color.Lime;
            this.chessGrid2.Location = new System.Drawing.Point(0, 40);
            this.chessGrid2.Name = "chessGrid2";
            this.chessGrid2.SelectedImage = new System.Drawing.Point(-1, -1);
            this.chessGrid2.SelectionColor = System.Drawing.Color.Purple;
            this.chessGrid2.SelectionLineWidth = 0.1F;
            this.chessGrid2.Size = new System.Drawing.Size(344, 192);
            this.chessGrid2.TabIndex = 1;
            this.chessGrid2.Selected += new ChessApplication.ImageGridEventHandler(this.chessGrid2_Selected);
            this.chessGrid2.SelectionChanged += new ChessApplication.ImageGridSelectionChangeEventHandler(this.chessGrid2_SelectionChanged);
            this.chessGrid2.SquareClick += new ChessApplication.ImageGridEventHandler(this.chessGrid2_SquareClick);
            this.chessGrid2.Dragged += new ChessApplication.ImageGridSelectionChangeEventHandler(this.chessGrid2_Dragged);
            this.chessGrid2.UnSelected += new ChessApplication.ImageGridEventHandler(this.chessGrid2_UnSelected);
            // 
            // ChessWindow
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(504, 320);
            this.Controls.Add(this.Captured2IG);
            this.Controls.Add(this.Captured1IG);
            this.Controls.Add(this.panel1);
            this.Controls.Add(this.chessGrid2);
            this.Controls.Add(this.statusBar1);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Menu = this.mainMenu1;
            this.Name = "ChessWindow";
            this.Text = "Emil Chess";
            this.Load += new System.EventHandler(this.ChessWindow_Load);
            this.Activated += new System.EventHandler(this.ChessWindow_Activated);
            this.Closing += new System.ComponentModel.CancelEventHandler(this.ChessWindow_Closing);
            this.Resize += new System.EventHandler(this.ChessWindow_Resize);
            this.panel1.ResumeLayout(false);
            this.panel1.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.highlightTimer)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args) 
		{
			if (args.Length > 0)
			{
				ChessWindow.FileToOpen = args[0];
			}
			else
				ChessWindow.FileToOpen = null;
			Application.Run(new ChessWindow());
		}

		private void statusBar1_PanelClick(object sender, System.Windows.Forms.StatusBarPanelClickEventArgs e)
		{
		
		}

		private void chessGrid1_Load(object sender, System.EventArgs e)
		{
		
		}

		private void ResizeWindow()
		{
			int windowWidth=ClientSize.Width;
			int windowHeight=ClientSize.Height - statusBar1.Height;
			int minPanelWidth=175,panelWidth=175;
			int chessGridSize, captureHeight;
			if (windowHeight*17>(windowWidth-minPanelWidth)*19)
			{
				chessGridSize = windowWidth-minPanelWidth;
				panelWidth = minPanelWidth;
				captureHeight = chessGridSize/17;
			}
			else
			{
				chessGridSize = windowHeight*17/19;
				captureHeight = windowHeight/19;
				panelWidth = windowWidth-chessGridSize;
			}
			chessGrid2.Size = new Size(chessGridSize,chessGridSize);
			Captured1IG.Size = new Size(chessGridSize,captureHeight);
			Captured2IG.Size = new Size(chessGridSize,captureHeight);
			Captured1IG.Location = new Point(0,0);
			chessGrid2.Location = new Point(0,Captured1IG.Height);
			Captured2IG.Location = new Point(0,Captured1IG.Height+chessGrid2.Height);


			/*chessGrid2.Width = chessGridSize;
			chessGrid2.Height = chessGridSize;*/
			panel1.Location = new Point(chessGridSize,0);
			panel1.Width = panelWidth;
			panel1.Height = windowHeight;
			WhitePlayerColorLabel.Width = panelWidth-WhitePlayerColorLabel.Location.X;
			BlackPlayerColorLabel.Width = panelWidth-BlackPlayerColorLabel.Location.X;
			nextPlayerLabel.Width = panelWidth-nextPlayerLabel.Location.X;
			historyListBox.Width = panelWidth;
			SendMsgTB.Width = panelWidth;
			MessagesTB.Width = panelWidth;
			if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				SendMsgTB.Location = new Point(0,panel1.Height-SendMsgTB.Height);
				//int sharedHeight = panel1.Height-SendMsgTB.Height-historyListBox.Location.Y;
				//historyListBox.Height = sharedHeight/2;
				historyListBox.Height = (panel1.Height-SendMsgTB.Height-historyListBox.Location.Y)/2;
				MessagesTB.Location = new Point(0,historyListBox.Location.Y+historyListBox.Height);
				MessagesTB.Height = SendMsgTB.Location.Y - MessagesTB.Location.Y;
			}
			else
				historyListBox.Height = windowHeight-historyListBox.Location.Y;
			//chessGrid2.Refresh();
		}

		private void ChessWindow_Resize(object sender, System.EventArgs e)
		{
			ResizeWindow();
		}

		private void ChessWindow_Load(object sender, System.EventArgs e)
		{
			//statusBar1.Text = rm.GetString("White_s_turn");;
			DisplayChessBoard();
		}

		private void chessGrid2_SquareClick(object sender, Point p)
		{

		}

		private void chessGrid2_Selected(object sender, Point p)
		{
			if (chessboard.GetPiece(GridPoint2ChessPosition(p)) == null)
			{
				chessGrid2.DoUnselect();
				chessGrid2.SetNoneHighlighted();
				ShowPossibleMovesForPieceMI.Enabled = true;
			}
			else
			{
				chessGrid2.SetNoneHighlighted();
				ShowPossibleMovesForPieceMI.Enabled = true;
			}
		}

		private void chessGrid2_UnSelected(object sender, Point p)
		{
			chessGrid2.SetNoneHighlighted();
			ShowPossibleMovesForPieceMI.Enabled = false;			
		}

		private void Moving(ChessMove move)
		{			
			ChessMoveErrorEnum res;
			ChessBoard boardCopy;
			ChessPlayerType nextPlayer;
			boardCopy = (ChessBoard)chessboard.Clone();
			res = chessboard.MovePiece(move,this.historyBoard);
			switch(res)
			{
				case ChessMoveErrorEnum.NoError:
					statusBar1.Text = "";
					ChangedSinceSaved = false;
				switch(chessboard.GameState) 
				{
					case ChessGameState.BlackInCheck:
						statusBar1.Text = BlackInCheckStr;
						break;
					case ChessGameState.WhiteInCheck:
						statusBar1.Text = WhiteInCheckStr;
						break;
					case ChessGameState.Draw:
						statusBar1.Text = GameEndedInDrawStr;
						break;
					case ChessGameState.BlackWon:
						statusBar1.Text = BlackWonStr;
						break;
					case ChessGameState.WhiteWon:
						statusBar1.Text = WhiteWonStr;
						break;
				}
					if (chessboard.ColorToMove == ChessPieceColor.Black)
						nextPlayer = BlackPlayer;
					else
						nextPlayer = WhitePlayer;
					if (nextPlayer != ChessPlayerType.HumanEmail)
					{
						historyListBox.Items.Add(chessboard.NumberOfMovePlayed.ToString("000") + ": " + move.From.ToString() + " -> " + move.To.ToString());
						if (historyListBox.Items.Count > 0)
							historyListBox.TopIndex = historyListBox.Items.Count-1;
						historyBoard.Push(boardCopy);
						historyMove.Push(move);
						enableControls();
					}
					/*if (!(BlackPlayer == ChessPlayerType.HumanIPNetworkClient
						|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
						|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
						|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer))
					{
						if (historyBoard.Count >= 1)
							CommandBackward1MoveMI.Enabled = true;
						if (historyBoard.Count >= 2)
							CommandBaclward2MovesMI.Enabled = true;
					}*/
#if (DEBUG)
					SaveGame("DebugGameSave" + chessboard.NumberOfMovePlayed.ToString("000") + ".cgs");
#endif
				switch (nextPlayer)
				{
					case ChessPlayerType.Computer:
						CallMoveGeneration(false);
						break;
					case ChessPlayerType.HumanEmail:						
						if (chessboard.ColorToMove == ChessPieceColor.Black)
						{
							if (SendGameByEmail(GameName, WhiteEmailAddress, BlackEmailAddress, SMTPServer))
							{
								historyListBox.Items.Add(chessboard.NumberOfMovePlayed.ToString("000") + ": " + move.From.ToString() + " -> " + move.To.ToString());
								if (historyListBox.Items.Count > 0)
									historyListBox.TopIndex = historyListBox.Items.Count-1;
								historyBoard.Push(boardCopy);
								historyMove.Push(move);
								enableControls();
							}
							else
							{
								chessboard = boardCopy;
							}
						}
						else
						{
							if (SendGameByEmail(GameName, BlackEmailAddress, WhiteEmailAddress, SMTPServer))
							{
								historyListBox.Items.Add(chessboard.NumberOfMovePlayed.ToString("000") + ": " + move.From.ToString() + " -> " + move.To.ToString());
								if (historyListBox.Items.Count > 0)
									historyListBox.TopIndex = historyListBox.Items.Count-1;
								historyBoard.Push(boardCopy);
								historyMove.Push(move);
								enableControls();
							}
							else
							{
								chessboard = boardCopy;
							}
						}							
						break;
					case ChessPlayerType.HumanIPNetworkClient:
						if (MyColor != chessboard.ColorToMove)
							EmilChessService.SetServerMove(move);
						break;
					case ChessPlayerType.HumanIPNetworkServer:
						if (MyColor != chessboard.ColorToMove)
							this.ChessService.SetClientMove(move);
						break;
				}
					break;
				case ChessMoveErrorEnum.NoPiece:
					statusBar1.Text = NoPieceSelectedStr;
					break;
				case ChessMoveErrorEnum.WrongSide:
					if (chessboard.ColorToMove == ChessPieceColor.Black)
					{
						statusBar1.Text = ChooseBlackInsteadStr;
					}
					else
					{
						statusBar1.Text = ChooseWhiteInsteadStr;
					}
					break;
				case ChessMoveErrorEnum.PutMyKingInCheck:
					if (chessboard.ColorToMove == ChessPieceColor.Black)
					{
						statusBar1.Text = WouldPutBlackInCheckStr;
					}
					else
					{
						statusBar1.Text = WouldPutWhiteInCheckStr;
					}
					break;
				case ChessMoveErrorEnum.GameHasEnded:
					break;
				case ChessMoveErrorEnum.IllegalMove:
					statusBar1.Text = IllegalMoveStr;
					break;
				case ChessMoveErrorEnum.IllegalDrawClaim:
					statusBar1.Text = IllegalDrawClaimStr;
					break;
			}
			DisplayChessBoard();
			chessGrid2.DoUnselect();
			chessGrid2.Refresh();
		}

		private void chessGrid2_Dragged(object sender, System.Drawing.Point from, System.Drawing.Point to)
		{
			if (!isGenerating)
			{
				if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
					|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
					|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
					|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
				{
					if (chessboard.GetPiece(GridPoint2ChessPosition(from)) != null)
					{
						if (chessboard.GetPiece(GridPoint2ChessPosition(from)).Color != MyColor)
						{
							statusBar1.Text = Illegal_move_not_my_turnStr;
							chessGrid2.DoUnselect();
							chessGrid2.Refresh();
							return;
						}
					}
				}
				Moving(new ChessMove(GridPoint2ChessPosition(from), GridPoint2ChessPosition(to)));
			}
		}

		private void chessGrid2_SelectionChanged(object sender, System.Drawing.Point from, System.Drawing.Point to)
		{		
			if (!isGenerating)
			{
				if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
					|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
					|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
					|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
				{
					if (chessboard.GetPiece(GridPoint2ChessPosition(from)) != null)
					{
						if (chessboard.GetPiece(GridPoint2ChessPosition(from)).Color != MyColor)
						{
							statusBar1.Text = Illegal_move_not_my_turnStr;
							chessGrid2.DoUnselect();
							chessGrid2.Refresh();
							return;
						}
					}
				}
				Moving(new ChessMove(GridPoint2ChessPosition(from), GridPoint2ChessPosition(to)));
			}
		}

		private void menuItem4_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
			ChessMove cm;			
			// Asynchronously invoke the GenerateMove method.
			IAsyncResult ar = GenerateMoveAsyncResult = engine.BeginDecideBestMove();
            cm = engine.EndDecideBestMove(GenerateMoveAsyncResult);
			//cm = chessboard.GenerateBestMove();
			//cm = chessboard.GenerateSimpleMove();
			Moving(cm);
		}

		private void label1_Click(object sender, System.EventArgs e)
		{
		
		}

		private void menuItem5_Click(object sender, System.EventArgs e)
		{
			CallMoveGeneration(false);
		}

		private void CallMoveGeneration(bool hintGeneration)
		{
			if (chessboard.GameState != ChessGameState.BlackWon
				&& chessboard.GameState != ChessGameState.WhiteWon
				&& chessboard.GameState != ChessGameState.Draw)
			{
				this.hintGeneration = hintGeneration;
				disableUI();
				//GenerateMoveAsyncResult = GenerateMoveCallback.BeginInvoke(null,null);
				GenerateMoveAsyncResult = engine.BeginDecideBestMove();
				ShowProgress = true;

				pollTimer.Start();
			}
		}

		private void disableUI()
		{
			isGenerating = true;
			chessGrid2.Enabled = false;
		}

		private void enableUI()
		{
			isGenerating = false;
			chessGrid2.Enabled = true;
		}

		private void pollTimer_Tick(object sender, System.EventArgs e)
		{
			if (GenerateMoveAsyncResult.IsCompleted)
			{
				pollTimer.Stop();
				ShowProgress = false;
				//ChessMove cm = GenerateMoveCallback.EndInvoke(GenerateMoveAsyncResult);
				ChessMove cm = engine.EndDecideBestMove(GenerateMoveAsyncResult);
				if (hintGeneration)
				{
					chessGrid2.SetNoneHighlighted();
					chessGrid2.SetHighlight(ChessPositionGridPoint(cm.From),true);
					chessGrid2.SetHighlight(ChessPositionGridPoint(cm.To),true);
					chessGrid2.Refresh();
					//MessageBox.Show(this,cm.ToString(),"Suggested move");
				}
				else
				{
					if (cm.DrawState == ChessDrawType.NoDraw)
					{
						chessGrid2.SetNoneHighlighted();
						chessGrid2.SetHighlight(ChessPositionGridPoint(cm.From),true);
						chessGrid2.SetHighlight(ChessPositionGridPoint(cm.To),true);
						chessGrid2.Refresh();
						highlightTimer.Enabled = true;
					}
					Moving(cm);
				}
				enableUI();
			}
			else
			{
				statusBar1.Text = NrMovesStr + ": " + engine.NumberOfGeneratedMoves.ToString("0000000");
				statusBar1.Refresh();
			}		
		}

		private void menuItem6_Click(object sender, System.EventArgs e)
		{
			ChessMove cm;
			cm = engine.DecideBestMove();
			Moving(cm);
		}

		private void FileNewGameMI_Click(object sender, System.EventArgs e)
		{
			//ChessAppNewGameDlg ngd = new ChessAppNewGameDlg(rm);
			DialogResult dr;
			dr = ngd.ShowDialog(this);
			if (dr == DialogResult.OK)
			{
				chessboard = new ChessBoard(BishopValue, KnightValue, PawnValue, QueenValue, RookValue);
				if (ngd.IsWhiteComputer())
					WhitePlayer = ChessPlayerType.Computer;
				else
				{
					WhitePlayer = ChessPlayerType.Human;
					MyColor = ChessPieceColor.White;
				}
				if (ngd.IsBlackComputer())
					BlackPlayer = ChessPlayerType.Computer;
				else
				{
					BlackPlayer = ChessPlayerType.Human;
					MyColor = ChessPieceColor.Black;
				}
				StartNewGame();
				if (ngd.GetGenerationMethod() == "AlphaBeta")
				{
					engine = new AlphaBetaEngine(chessboard,historyBoard,ngd.GetSearchDepth());
				} 
				else if (ngd.GetGenerationMethod() == "SearchNBest")
				{
					engine = new SearchNBestEngine(chessboard,historyBoard,ngd.GetSearchDepth(),ngd.GetSearchNodes());
				}
				else if (ngd.GetGenerationMethod() == "Simple")
				{
					engine = new SimpleEngine(chessboard,historyBoard,ngd.GetSearchDepth());
				}
				else
				{
					engine = new SimpleEngine(chessboard,historyBoard,ngd.GetSearchDepth());
				}
				ConfigureUI();
				if (WhitePlayer == ChessPlayerType.Computer)
					CallMoveGeneration(false);
			}
		}

		private void CommandSuggestMoveMI_Click(object sender, System.EventArgs e)
		{
			CallMoveGeneration(true);		
		}

		private void HelpAboutMI_Click(object sender, System.EventArgs e)
		{
			ChessAppAbout d = new ChessAppAbout(rm);
			d.ShowDialog(this);			
		}

		private void ChessWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (!ChangedSinceSaved)
			{
				if (DialogResult.No == MessageBox.Show(SureOfQuitingStr, SureOfQuiting_titleStr,
					MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, 
					MessageBoxDefaultButton.Button2))
					e.Cancel = true;
			}
		}

		private void SaveGame(string fileName)
		{
			try
			{
				SaveGame(new XmlTextWriter(fileName,new System.Text.UTF8Encoding()));
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,Error_saving_gameStr + " \"" + fileName + "\"" + System.Environment.NewLine + ex.Message);
			}
		}
        /// <summary>
        /// Save the game state to a XML representation
        /// </summary>
        /// <returns> Returns a string containing the XML representation of the game state</returns>
		public string SaveGameToXMLString()
		{
			StringWriter sw=new StringWriter();
			SaveGame(new XmlTextWriter(sw));
			return sw.ToString();
		}

		private void SaveGame(XmlTextWriter xmlW)
		{
			xmlW.Formatting = Formatting.Indented; 
			xmlW.WriteStartDocument();
			xmlW.WriteStartElement("ChessGameSave");
			xmlW.WriteElementString("Version",Assembly.GetExecutingAssembly().GetName().Version.ToString());
			//xmlW.WriteElementString("hasComputerPlayer",XmlConvert.ToString(hasComputerPlayer));
			//xmlW.WriteElementString("computerSide",XmlConvert.ToString((int)computerSide));
			xmlW.WriteElementString("BlackPlayer",XmlConvert.ToString((int)BlackPlayer));
			xmlW.WriteElementString("WhitePlayer",XmlConvert.ToString((int)WhitePlayer));
			chessboard.SaveToXml(xmlW);
			if (BlackPlayer == ChessPlayerType.Computer || WhitePlayer == ChessPlayerType.Computer)
			{
				xmlW.WriteStartElement("MoveEngine");
				engine.SaveToXml(xmlW);
				xmlW.WriteEndElement();
			}
			else if (BlackPlayer == ChessPlayerType.HumanEmail || WhitePlayer == ChessPlayerType.HumanEmail)
			{
				xmlW.WriteStartElement("EmailGameInfo");
				xmlW.WriteElementString("BlackEmailAddress",BlackEmailAddress);
				xmlW.WriteElementString("WhiteEmailAddress",WhiteEmailAddress);
				xmlW.WriteElementString("GameName",GameName);
				xmlW.WriteEndElement();
			}
			else if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient 
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				xmlW.WriteStartElement("NetworkGameInfo");
				xmlW.WriteElementString("ListenPort",XmlConvert.ToString((int)ListenPort));
				xmlW.WriteEndElement();				
			}
			xmlW.WriteStartElement("MoveHistory");
			ArrayList moves = new ArrayList(historyMove);
			moves.Reverse();
			foreach (ChessMove move in moves)
			{
				move.SaveToXml(xmlW);
			}
			xmlW.WriteEndElement();

			xmlW.WriteStartElement("BoardHistory");
			ArrayList boards = new ArrayList(historyBoard);
			boards.Reverse();
			int ToSave=Math.Min(8,boards.Count);
			for(int i=0; i<ToSave; i++)
			{
				((ChessBoard)boards[i]).SaveToXml(xmlW);
			}
			xmlW.WriteEndElement();

			xmlW.WriteEndElement();
			xmlW.WriteEndDocument();
			//Write the XML to file and close the writer
			xmlW.Flush();
			xmlW.Close();
			ChangedSinceSaved = true;
		}

		private void FileSaveGameMI_Click(object sender, System.EventArgs e)
		{
			if (DialogResult.OK == saveFileDialog1.ShowDialog(this))
			{
				SaveGame(saveFileDialog1.FileName);
				ChangedSinceSaved = true;
			}
		}

		private void LoadGame(string fileName)
		{
			try 
			{
				LoadGame(new XmlTextReader(fileName),true);
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,Error_loading_gameStr + " \"" + fileName + "\"" + System.Environment.NewLine + ex.Message);
			}
		}

		/// <summary>
		/// Load the game state from a XML representation.
		/// </summary>
		/// <param name="GameXMLStr"> A string containing the XML representation of the game state</param>
		public void LoadGameFromXMLString(string GameXMLStr)
		{
			//Create the XmlParserContext.
			//XmlParserContext context = new XmlParserContext(null, nsmgr, null, XmlSpace.None);

			//Create the reader. 
			XmlTextReader reader = new XmlTextReader(GameXMLStr, XmlNodeType.Element, null);
			LoadGame(reader,false);
		}

		private void LoadGame(XmlTextReader xmlR, bool ServerLoad)
		{
			xmlR.Read();
			if (xmlR.NodeType != XmlNodeType.XmlDeclaration) throw new ApplicationException("No XML declaration node");

			xmlR.ReadStartElement("ChessGameSave");
			while (true)
				if (xmlR.IsStartElement("Version")) 
				{
					xmlR.ReadString();
					xmlR.ReadEndElement();
				}
				else if (xmlR.IsStartElement("ChessBoard")) 
				{
					chessboard = new ChessBoard(xmlR);
				}
				else if (xmlR.IsStartElement("BlackPlayer")) 
				{
					BlackPlayer = (ChessPlayerType)XmlConvert.ToInt32(xmlR.ReadString());
					xmlR.ReadEndElement();
				}
				else if (xmlR.IsStartElement("WhitePlayer")) 
				{
					WhitePlayer = (ChessPlayerType)XmlConvert.ToInt32(xmlR.ReadString());
					xmlR.ReadEndElement();
				}
				else if (xmlR.IsStartElement("MoveEngine")) 
				{
					xmlR.ReadStartElement("MoveEngine");
					while (xmlR.NodeType != XmlNodeType.Element)
						xmlR.Skip();
					if (xmlR.NodeType == XmlNodeType.Element)
					{
						string EngineTypeStr = "ChessApplication" + "." + xmlR.Name;
						Type EngineType = Type.GetType(EngineTypeStr);
						engine = (ChessEngine)Activator.CreateInstance(EngineType,new Object[3]{xmlR,chessboard,historyBoard});
						xmlR.ReadEndElement();
					}
					else
						engine = null;
				}
				else if (xmlR.IsStartElement("EmailGameInfo")) 
				{
					xmlR.ReadStartElement("EmailGameInfo");
					
					xmlR.ReadStartElement("BlackEmailAddress");
					BlackEmailAddress = xmlR.ReadString();
					xmlR.ReadEndElement();
					
					xmlR.ReadStartElement("WhiteEmailAddress");
					WhiteEmailAddress = xmlR.ReadString();
					xmlR.ReadEndElement();

					xmlR.ReadStartElement("GameName");
					GameName = xmlR.ReadString();
					xmlR.ReadEndElement();

					xmlR.ReadEndElement();
				}
				else if (xmlR.IsStartElement("NetworkGameInfo")) 
				{
					xmlR.ReadStartElement("NetworkGameInfo");
					
					xmlR.ReadStartElement("ListenPort");
					ListenPort = XmlConvert.ToInt32(xmlR.ReadString());
					xmlR.ReadEndElement();

					xmlR.ReadEndElement();
				}
				else if (xmlR.IsStartElement("MoveHistory")) 
				{
					if (xmlR.IsEmptyElement)
						xmlR.ReadStartElement("MoveHistory");
					else
					{
						xmlR.ReadStartElement("MoveHistory");
						ChessMove cm;
						int cnt=1;
						historyListBox.Items.Clear();
						while (xmlR.IsStartElement("ChessMove"))
						{
							cm = new ChessMove(xmlR);
							historyListBox.Items.Add(cnt.ToString("000") + ": " + cm.From.ToString() + " -> " + cm.To.ToString());
							cnt++;
							historyMove.Push(cm);
						}												
						xmlR.ReadEndElement();
					}
				}
				else if (xmlR.IsStartElement("BoardHistory"))
				{
					if (xmlR.IsEmptyElement)
						xmlR.ReadStartElement("BoardHistory");
					else
					{
						xmlR.ReadStartElement("BoardHistory");
						historyBoard = new Stack();
						ChessBoard NewBoard;
						while (xmlR.IsStartElement("ChessBoard"))
						{
							NewBoard = new ChessBoard(xmlR);
							historyBoard.Push(NewBoard);
							NewBoard = null;
						}						
						xmlR.ReadEndElement();
					}
				}
				else if (((xmlR.NodeType == XmlNodeType.EndElement) && (xmlR.Name == "ChessGameSave")) || xmlR.EOF) 
				{
					break;
				} 
				else
				{
					xmlR.Read();
					//throw new ApplicationException("Unknown node found");
				}
			//GenerateMoveCallback = new GenerateMoveDelegate(chessboard.DecideMove);
			if (engine == null)
				engine = new AlphaBetaEngine(chessboard,historyBoard,4);
			engine.ForBoard = chessboard;
			engine.BoardHistoryStack = this.historyBoard;
			if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				if (ServerLoad)
				{
					ChessAppNewGameAsServer cgs = new ChessAppNewGameAsServer(rm,this,ServerLoad,HelpFilePath);
					DialogResult dr;
					if (BlackPlayer == ChessPlayerType.HumanIPNetworkServer)
						MyColor = ChessPieceColor.Black;
					else
						MyColor = ChessPieceColor.White;
					cgs.MyColor = MyColor;
					cgs.Port = ListenPort;

					dr = cgs.ShowDialog(this);
					if (dr == DialogResult.OK)
					{
						NetworkTimer.Enabled = true;
					}
					else
					{
						// Do something when i didn't get a connection
						MessageBox.Show(this,"Restart of networked game failed!");
					}
				}
			}
			else if (BlackPlayer == ChessPlayerType.HumanEmail
				|| WhitePlayer == ChessPlayerType.HumanEmail)
			{
				if (chessboard.ColorToMove == ChessPieceColor.Black)
					MyColor = ChessPieceColor.Black;
				else
					MyColor = ChessPieceColor.White;			
				ConfigureUI();
			} 
			else 
			{
				if (BlackPlayer == ChessPlayerType.Computer)
                    MyColor = ChessPieceColor.White;
				else
					MyColor = ChessPieceColor.Black;
				ConfigureUI();
				if (chessboard.ColorToMove == ChessPieceColor.Black)
				{
					if (BlackPlayer == ChessPlayerType.Computer)
						CallMoveGeneration(false);					
				}
				else
				{
					if (WhitePlayer == ChessPlayerType.Computer)
						CallMoveGeneration(false);
				}
			}
			DisplayChessBoard();
			this.Refresh();
			ChangedSinceSaved = true;
		}			

		private void FileLoadGameMI_Click(object sender, System.EventArgs e)
		{
			if (DialogResult.OK == openFileDialog1.ShowDialog(this))
			{
				// Opens a file and serializes the object into it in binary format.
				/*XmlDocument doc = new XmlDocument();
				doc.Load(openFileDialog1.FileName);
				XmlNodeReader reader = new XmlNodeReader(doc);*/
				historyBoard = new Stack();

				LoadGame(openFileDialog1.FileName);
			}			
		}

		private void stepBack(int steps)
		{
			ChessBoard previousBoard = null;
			ChessPlayerType nextPlayer;
			int nr;
			if (steps > historyBoard.Count)
			{
				MessageBox.Show(UnableToStepBackStr);
				return ;
			}
			for(int i=0; i<steps; ++i)
			{
				previousBoard = (ChessBoard)historyBoard.Pop();
				historyMove.Pop();
			}
			chessboard = previousBoard;
			engine.ForBoard = chessboard;
			historyListBox.Items.Clear();
			nr = historyMove.Count;			
			foreach (ChessMove cm in historyMove)
			{
				historyListBox.Items.Insert(0,nr.ToString("000") + ": " + cm.From.ToString() + " -> " + cm.To.ToString());
				nr--;
			}
			if (historyListBox.Items.Count > 0)
				historyListBox.TopIndex = historyListBox.Items.Count-1;
			if (historyBoard.Count >= 1)
				CommandBackward1MoveMI.Enabled = true;
			else 
				CommandBackward1MoveMI.Enabled = false;
			if (historyBoard.Count >= 2)
				CommandBaclward2MovesMI.Enabled = true;
			else
				CommandBaclward2MovesMI.Enabled = false;
			if (chessboard.ColorToMove == ChessPieceColor.Black)
				nextPlayer = BlackPlayer;
			else
				nextPlayer = WhitePlayer;
			if (nextPlayer == ChessPlayerType.Computer)
				CallMoveGeneration(false);
			DisplayChessBoard();
			chessGrid2.DoUnselect();
			chessGrid2.Refresh();
		}

		private void CommandBackward1MoveMI_Click(object sender, System.EventArgs e)
		{
			stepBack(1);		
		}

		private void CommandBaclward2MovesMI_Click(object sender, System.EventArgs e)
		{
            stepBack(2);		
		}

		private void ClaimDraw3StateRepeatRuleMI_Click(object sender, System.EventArgs e)
		{
			Moving(ChessMove.ClaimDrawRuleRepetition());
			/*if (chessboard.DrawMoveRepetition(this.historyBoard))
				MessageBox.Show("Moves have been repeated");
			else
				MessageBox.Show("Moves have NOT been repeated");*/
		}

		private void Claim50MovesRuleMI_Click(object sender, System.EventArgs e)
		{
			Moving(ChessMove.ClaimDrawRule50Move());
		}

		private void highlightTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			chessGrid2.SetNoneHighlighted();
			highlightTimer.Enabled = false;
			chessGrid2.Refresh();
		}

		private void ShowPossibleMovesForPieceMI_Click(object sender, System.EventArgs e)
		{
			Queue q;
			if (chessGrid2.SelectedImage.X>=0 && chessGrid2.SelectedImage.Y>=0)
			{
				q = chessboard.GenerateNewPositionsForPiece(GridPoint2ChessPosition(chessGrid2.SelectedImage));
				if (q.Count > 0)
				{
					chessGrid2.SetNoneHighlighted();
					foreach(ChessPosition cp in q)
					{
						chessGrid2.SetHighlight(this.ChessPositionGridPoint(cp),true);
					}
					chessGrid2.Refresh();
				}
			}			
		}
		private Color ExtractColor(XmlElement e)
		{
			Color c = Color.FromArgb(XmlConvert.ToByte(e.GetAttribute("Red")),
				XmlConvert.ToByte(e.GetAttribute("Green")),
				XmlConvert.ToByte(e.GetAttribute("Blue")));
			return c;
		}
		private Font ExtractFont(XmlElement e, Font standardFont)
		{
			try 
			{
				Font f = new Font(e.GetAttribute("FontName"),
					XmlConvert.ToSingle(e.GetAttribute("FontSize")));
				return f;
			}
			catch (Exception)
			{
				return standardFont;
			}
		}
		private void LoadSkin(string skinPath)
		{
			try 
			{
				if (skinPath.Substring(0,5) == "Skins")
					skinPath = ProgramDir + @"\" + skinPath;
				XmlReader xmlR = new XmlTextReader(skinPath);
				XmlValidatingReader vr = new XmlValidatingReader(xmlR);
				XmlDocument xmldoc = new XmlDocument();
				vr.Schemas.Add(null, DataDir + @"\Skin.xsd");
				xmldoc.Load(vr);
				XmlElement root = xmldoc.DocumentElement;
				XmlNode node;
				XmlElement element;
				DirectoryInfo di=Directory.GetParent(skinPath);
				string skinDir=di.FullName;
				Color TransparentColor;
				string imagePath;

				node = root.SelectSingleNode("/Skin/PieceSet/BlackBishopImage");
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackBishopImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/BlackKingImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackKingImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/BlackKnightImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackKnightImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/BlackPawnImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackPawnImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/BlackQueenImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackQueenImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/BlackRookImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				BlackRookImage = Image.FromFile(imagePath);

				node = root.SelectSingleNode("/Skin/PieceSet/WhiteBishopImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhiteBishopImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/WhiteKingImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhiteKingImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/WhiteKnightImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhiteKnightImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/WhitePawnImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhitePawnImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/WhiteQueenImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhiteQueenImage = Image.FromFile(imagePath);
				node = root.SelectSingleNode("/Skin/PieceSet/WhiteRookImage") as XmlElement;
				imagePath = skinDir + "\\" + node.FirstChild.Value;
				WhiteRookImage = Image.FromFile(imagePath);

				node = root.SelectSingleNode("/Skin/PieceSet/TransparentColor");
				if (node != null)
				{
					TransparentColor = ExtractColor(node as XmlElement);
					if (BlackBishopImage is Bitmap)
						((Bitmap)BlackBishopImage).MakeTransparent(TransparentColor);
					if (BlackKingImage is Bitmap)
						((Bitmap)BlackKingImage).MakeTransparent(TransparentColor);
					if (BlackKnightImage is Bitmap)
						((Bitmap)BlackKnightImage).MakeTransparent(TransparentColor);
					if (BlackPawnImage is Bitmap)
						((Bitmap)BlackPawnImage).MakeTransparent(TransparentColor);
					if (BlackQueenImage is Bitmap)
						((Bitmap)BlackQueenImage).MakeTransparent(TransparentColor);
					if (BlackRookImage is Bitmap)
						((Bitmap)BlackRookImage).MakeTransparent(TransparentColor);
					if (WhiteBishopImage is Bitmap)
						((Bitmap)WhiteBishopImage).MakeTransparent(TransparentColor);
					if (WhiteKingImage is Bitmap)
						((Bitmap)WhiteKingImage).MakeTransparent(TransparentColor);
					if (WhiteKnightImage is Bitmap)
						((Bitmap)WhiteKnightImage).MakeTransparent(TransparentColor);
					if (WhitePawnImage is Bitmap)
						((Bitmap)WhitePawnImage).MakeTransparent(TransparentColor);	
					if (WhiteQueenImage is Bitmap)
						((Bitmap)WhiteQueenImage).MakeTransparent(TransparentColor);
					if (WhiteRookImage is Bitmap)
						((Bitmap)WhiteRookImage).MakeTransparent(TransparentColor);
				}

				element = root.SelectSingleNode("/Skin/Colors/Black") as XmlElement;
				BlackBrush = new SolidBrush(ExtractColor(element));
				chessGrid2.DarkBackBrush = BlackBrush;
				element = root.SelectSingleNode("/Skin/Colors/White") as XmlElement;
				WhiteBrush = new SolidBrush(ExtractColor(element));
				chessGrid2.LightBackBrush = WhiteBrush;
				if (ColorAtBottom == ChessPieceColor.Black)
				{
					Captured2IG.DarkBackBrush = BlackBrush;
					Captured2IG.LightBackBrush = BlackBrush; 
					Captured1IG.DarkBackBrush = WhiteBrush;
					Captured1IG.LightBackBrush = WhiteBrush;
				}
				else
				{
					Captured1IG.DarkBackBrush = BlackBrush;
					Captured1IG.LightBackBrush = BlackBrush; 
					Captured2IG.DarkBackBrush = WhiteBrush;
					Captured2IG.LightBackBrush = WhiteBrush;
				}
				element = root.SelectSingleNode("/Skin/Colors/Selection") as XmlElement;
				chessGrid2.SelectionColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/Highlight") as XmlElement;
				chessGrid2.HighlightingColor = ExtractColor(element);

				element = root.SelectSingleNode("/Skin/Colors/WindowBack") as XmlElement;
				WindowBack = ExtractColor(element);
				this.BackColor = WindowBack;
				panel1.BackColor = WindowBack;
				element = root.SelectSingleNode("/Skin/Colors/WindowFore") as XmlElement;
				WindowFore = ExtractColor(element);
				this.ForeColor = WindowFore;
				panel1.ForeColor = WindowFore;
			

				//mainMenu1.Container.ForeColor = WindowFore;
			            
				/*        <MenyBack Red="255" Green="255" Blue="255" />
						<MenyFore Red="0" Green="0" Blue="0" />*/

				element = root.SelectSingleNode("/Skin/Colors/GridBack") as XmlElement;
				chessGrid2.BackBrush = new SolidBrush(ExtractColor(element));
				element = root.SelectSingleNode("/Skin/Colors/GridFore") as XmlElement;
				chessGrid2.ForeBrush = new SolidBrush(ExtractColor(element));

				element = root.SelectSingleNode("/Skin/Colors/StatusBarBack") as XmlElement;
				statusBar1.BackColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/StatusBarFore") as XmlElement;
				StatusBarFontColor = ExtractColor(element);
				statusBar1.ForeColor = StatusBarFontColor;
				element = root.SelectSingleNode("/Skin/Colors/StatusBarProgressBar") as XmlElement;
				StatusBarProgressBarColor = ExtractColor(element);

				element = root.SelectSingleNode("/Skin/Colors/HistoryListBack") as XmlElement;
				historyListBox.BackColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/HistoryListFore") as XmlElement;
				historyListBox.ForeColor = ExtractColor(element);
			
				element = root.SelectSingleNode("/Skin/Colors/ChatHistoryBack") as XmlElement;
				MessagesTB.BackColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/ChatHistoryFore") as XmlElement;
				MessagesTB.ForeColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/ChatSendBack") as XmlElement;
				SendMsgTB.BackColor = ExtractColor(element);
				element = root.SelectSingleNode("/Skin/Colors/ChatSendFore") as XmlElement;
				SendMsgTB.ForeColor = ExtractColor(element);

				// Loading Fonts
				element = root.SelectSingleNode("/Skin/Fonts/Window") as XmlElement;
				this.Font = ExtractFont(element,this.Font);
				panel1.Font = ExtractFont(element,panel1.Font);
				element = root.SelectSingleNode("/Skin/Fonts/Grid") as XmlElement;
				chessGrid2.Font = ExtractFont(element,chessGrid2.Font);
				element = root.SelectSingleNode("/Skin/Fonts/StatusBar") as XmlElement;
				statusBar1.Font = ExtractFont(element,statusBar1.Font);
				element = root.SelectSingleNode("/Skin/Fonts/HistoryList") as XmlElement;
				historyListBox.Font = ExtractFont(element,historyListBox.Font);
				element = root.SelectSingleNode("/Skin/Fonts/ChatHistory") as XmlElement;
				MessagesTB.Font = ExtractFont(element,MessagesTB.Font);
				element = root.SelectSingleNode("/Skin/Fonts/ChatSend") as XmlElement;
				SendMsgTB.Font = ExtractFont(element,SendMsgTB.Font);
				//WindowBack = ExtractColor(element);
				//this.BackColor = WindowBack;
				//panel1.BackColor = WindowBack;

				//DisplayChessBoard();
				//this.chessGrid2.Refresh();
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,Error_loading_skinStr + " \"" + skinPath + "\"" + System.Environment.NewLine + ex.Message);				
			}
		}

		private void LoadConfig()
		{
			try 
			{
				XmlTextReader xmlR = new XmlTextReader(DataDir + @"\Config.xml");
				XmlValidatingReader vr = new XmlValidatingReader(xmlR);			
				vr.Schemas.Add(null, DataDir + @"\Config.xsd");
				ConfigXmlDoc = new XmlDocument();
				ConfigXmlDoc.Load(vr);
				XmlNode node;
				//Create an XmlNamespaceManager for resolving namespaces.
				//XmlNamespaceManager nsmgr = new XmlNamespaceManager(ConfigXmlDoc.NameTable);
				//nsmgr.AddNamespace("","urn:Config");

				//XmlElement element;
			
				XmlElement root = ConfigXmlDoc.DocumentElement;

				node = root.SelectSingleNode("/Configuration/ValueOfBishop");
				BishopValue = XmlConvert.ToDouble(node.FirstChild.Value);
				node = root.SelectSingleNode("/Configuration/ValueOfKnight");
				KnightValue = XmlConvert.ToDouble(node.FirstChild.Value);
				node = root.SelectSingleNode("/Configuration/ValueOfPawn");
				PawnValue = XmlConvert.ToDouble(node.FirstChild.Value);
				node = root.SelectSingleNode("/Configuration/ValueOfQueen");
				QueenValue = XmlConvert.ToDouble(node.FirstChild.Value);
				node = root.SelectSingleNode("/Configuration/ValueOfRook");
				RookValue = XmlConvert.ToDouble(node.FirstChild.Value);

				node = root.SelectSingleNode("/Configuration/ValueOfRook");
				highlightTimer.Interval = 1000*XmlConvert.ToInt32(node.FirstChild.Value);

				node = root.SelectSingleNode("/Configuration/Skin");
				SkinPath = node.FirstChild.Value;
				node = root.SelectSingleNode("/Configuration/ServicePort");
				ServicePort = XmlConvert.ToInt32(node.FirstChild.Value);
				node = root.SelectSingleNode("/Configuration/EmailAddress");
				if (node.FirstChild == null)
					EmailAddress = "";
				else
					EmailAddress = node.FirstChild.Value;

				node = root.SelectSingleNode("/Configuration/SMTPServer");
				if (node.FirstChild == null)
					SMTPServer = "";
				else
					SMTPServer = node.FirstChild.Value;

				node = root.SelectSingleNode("/Configuration/LanguageCode");
				if (node.FirstChild == null)
					LanguageCode = "";
				else
					LanguageCode = node.FirstChild.Value;

				xmlR.Close();

				LoadSkin(SkinPath);
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,Error_loading_configuration_fileStr + " \"" + DataDir + @"\Config.xml" + "\"" + System.Environment.NewLine + ex.Message);
			}
		}

		private void SaveConfig()
		{
			try 
			{
				XmlNode node;
				//XmlElement element;
				XmlElement root = ConfigXmlDoc.DocumentElement;

				node = root.SelectSingleNode("/Configuration/ValueOfBishop");
				node.FirstChild.Value = XmlConvert.ToString(BishopValue);
				node = root.SelectSingleNode("/Configuration/ValueOfKnight");
				node.FirstChild.Value = XmlConvert.ToString(KnightValue);
				node = root.SelectSingleNode("/Configuration/ValueOfPawn");
				node.FirstChild.Value = XmlConvert.ToString(PawnValue);
				node = root.SelectSingleNode("/Configuration/ValueOfQueen");
				node.FirstChild.Value = XmlConvert.ToString(QueenValue);
				node = root.SelectSingleNode("/Configuration/ValueOfRook");
				node.FirstChild.Value = XmlConvert.ToString(RookValue);

				node = root.SelectSingleNode("/Configuration/ValueOfRook");
				node.FirstChild.Value = XmlConvert.ToString(highlightTimer.Interval/1000);

				node = root.SelectSingleNode("/Configuration/Skin");
				node.FirstChild.Value = SkinPath;

				node = root.SelectSingleNode("/Configuration/ServicePort");
				node.FirstChild.Value = XmlConvert.ToString(ServicePort);
				node = root.SelectSingleNode("/Configuration/EmailAddress");
				if (node.FirstChild == null)
				{
					//node.AppendChild(ConfigXmlDoc.CreateNode(XmlNodeType.Text,   (XmlNodeType.Element, "pages", "");  
					node.InnerText = EmailAddress;
				}
				else
					node.FirstChild.Value = EmailAddress;
				node = root.SelectSingleNode("/Configuration/SMTPServer");
				if (node.FirstChild == null)
					node.InnerText = SMTPServer;
				else
					node.FirstChild.Value = SMTPServer;

				node = root.SelectSingleNode("/Configuration/LanguageCode");
				if (node.FirstChild == null)
					node.InnerText = LanguageCode;
				else
					node.FirstChild.Value = LanguageCode;

				ConfigXmlDoc.Save(DataDir + @"\Config.xml");
			}
			catch (Exception ex)
			{
				MessageBox.Show(this,Error_saving_configuration_fileStr + " \"" + DataDir + @"\Config.xml" + "\"" + System.Environment.NewLine + ex.Message);
			}
		}

		private void FileOptionsMI_Click(object sender, System.EventArgs e)
		{
			ChessAppOptions optionsDlg = new ChessAppOptions(rm,HelpFilePath);
			DialogResult dr;
			optionsDlg.BishopValue = BishopValue;
			optionsDlg.KnightValue = KnightValue;
			optionsDlg.PawnValue = PawnValue;
			optionsDlg.QueenValue = QueenValue;
			optionsDlg.RookValue = RookValue;

			optionsDlg.MoveHightlightSec = (int)highlightTimer.Interval/1000;

			optionsDlg.ServicePort = ServicePort;
			optionsDlg.EmailAddress = EmailAddress;
			optionsDlg.SMTPServer = SMTPServer;

			optionsDlg.LanguageCode = LanguageCode;
				
			optionsDlg.SkinPath = SkinPath;
			dr = optionsDlg.ShowDialog(this);
			if (dr == DialogResult.OK)
			{
				BishopValue = optionsDlg.BishopValue;				
				KnightValue = optionsDlg.KnightValue;				
				PawnValue = optionsDlg.PawnValue;				
				QueenValue = optionsDlg.QueenValue;				
				RookValue = optionsDlg.RookValue;

				highlightTimer.Interval = 1000*optionsDlg.MoveHightlightSec;

				ServicePort = optionsDlg.ServicePort;
                EmailAddress = optionsDlg.EmailAddress;
				SMTPServer = optionsDlg.SMTPServer;

				LanguageCode = optionsDlg.LanguageCode;
				
				SkinPath = optionsDlg.SkinPath;

				chessboard.ApplyPieceValues(BishopValue, KnightValue,
					PawnValue, QueenValue, RookValue);

				SaveConfig();

				LoadSkin(SkinPath);
			}
		}

		private void NewNetworkGameMI_Click(object sender, System.EventArgs e)
		{
			ChessAppNewGameAsServer cgs = new ChessAppNewGameAsServer(rm,this,false,HelpFilePath);
			DialogResult dr;

			cgs.Port = this.ServicePort;
			
			dr = cgs.ShowDialog(this);			
			if (dr == DialogResult.OK)
			{
				MyColor = cgs.MyColor;
				if (MyColor == ChessPieceColor.Black)
				{
					BlackPlayer = ChessPlayerType.HumanIPNetworkServer;
					WhitePlayer = ChessPlayerType.HumanIPNetworkClient;
				}
				else
				{
					BlackPlayer = ChessPlayerType.HumanIPNetworkClient;
					WhitePlayer = ChessPlayerType.HumanIPNetworkServer;
				}
				NetworkTimer.Enabled = true;
			}
		}

		private void statusBar1_DrawItem(object sender, System.Windows.Forms.StatusBarDrawItemEventArgs sbdevent)
		{
			SolidBrush sb=new SolidBrush(statusBar1.ForeColor);
			SolidBrush sbb=new SolidBrush(statusBar1.BackColor);
			SolidBrush sbp=new SolidBrush(StatusBarProgressBarColor);
			Rectangle progressRect=sbdevent.Bounds;

			//StatusBarFontColor
			//sbdevent.DrawBackground();
			sbdevent.Graphics.FillRectangle(sbb,sbdevent.Bounds);
			if (ShowProgress)
			{
				progressRect.Height = progressRect.Height-2;
				progressRect.Y = progressRect.Y+1;
				if (engine.NumberOfGeneratedMoves < engine.ApproxNrNodes())
					progressRect.Width = progressRect.Width*engine.NumberOfGeneratedMoves/engine.ApproxNrNodes();
				sbdevent.Graphics.FillRectangle(sbp,progressRect);
			}
			sbdevent.Graphics.DrawString(statusBar1.Text, sbdevent.Font,sb,sbdevent.Bounds);

			//statusBar1.St
			
		}
		/// <summary>
		/// Update what the controllers used and strings displayed.
		/// </summary>
		public void ConfigureUI()
		{
			enableControls();
			BlackPlayerColorLabel.Text = ChessPlayerTypeStrings[(int)BlackPlayer];
			WhitePlayerColorLabel.Text = ChessPlayerTypeStrings[(int)WhitePlayer];;			
			if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				if (MyColor == ChessPieceColor.Black)
					BlackPlayerColorLabel.Text = MeStr;
				else
					WhitePlayerColorLabel.Text = MeStr;
			}
			else if (BlackPlayer == ChessPlayerType.HumanEmail
				|| WhitePlayer == ChessPlayerType.HumanEmail)				
			{
				if (MyColor == ChessPieceColor.Black)
					BlackPlayerColorLabel.Text = MeStr;
				else
					WhitePlayerColorLabel.Text = MeStr;
			}
			else
			{

			}
			DecideColorAtBottom();
			ResizeWindow();
			//this.Refresh();
		}

		private void enableControls()
		{
			FileSaveGameMI.Enabled = true;
			FileLoadGameMI.Enabled = true;
			CommandBackward1MoveMI.Enabled = true;
			CommandBaclward2MovesMI.Enabled = true;
			chessGrid2.Enabled = true;
			if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				SendMsgTB.Visible = true;
				MessagesTB.Visible = true;
				if (MyColor == ChessPieceColor.Black)
					BlackPlayerColorLabel.Text = MeStr;
				else
					WhitePlayerColorLabel.Text = MeStr;
				ChessPlayerType myPlayer;
				if (MyColor == ChessPieceColor.Black)
					myPlayer = BlackPlayer;
				else
					myPlayer = WhitePlayer;
				CommandBackward1MoveMI.Enabled = false;
				CommandBaclward2MovesMI.Enabled = false;
				if (myPlayer == ChessPlayerType.HumanIPNetworkClient)
				{
					FileSaveGameMI.Enabled = false;
					FileLoadGameMI.Enabled = false;
				}
			}
			else if (BlackPlayer == ChessPlayerType.HumanEmail
				|| WhitePlayer == ChessPlayerType.HumanEmail)				
			{
				SendMsgTB.Visible = false;
				MessagesTB.Visible = false;
				CommandBackward1MoveMI.Enabled = false;
				CommandBaclward2MovesMI.Enabled = false;
				if (MyColor == chessboard.ColorToMove)
					chessGrid2.Enabled = true;
				else
					chessGrid2.Enabled = false;
			}
			else
			{
				SendMsgTB.Visible = false;
				MessagesTB.Visible = false;
				chessGrid2.Enabled = true;
				if (historyBoard.Count >= 1)
					CommandBackward1MoveMI.Enabled = true;
				if (historyBoard.Count >= 2)
					CommandBaclward2MovesMI.Enabled = true;
			}
		}
		/// <summary>
		/// Start a new game i.e. Reset game, remove history etc.
		/// It does not set what player or update the controllers to use.
		/// </summary>
		public void StartNewGame()
		{	
			chessboard = new ChessBoard(BishopValue, KnightValue, PawnValue, QueenValue, RookValue);
			historyBoard.Clear();
			historyMove.Clear();
			DisplayChessBoard();
			chessGrid2.DoUnselect();
			chessGrid2.Refresh();
			historyListBox.Items.Clear();
		}

		private void FileNewGameClientMI_Click(object sender, System.EventArgs e)
		{
			ChessAppNewGameAsClient cgc = new ChessAppNewGameAsClient(rm,HelpFilePath);
			DialogResult dr;
			string GameStateXML;
			dr = cgc.ShowDialog(this);
			if (dr == DialogResult.OK)
			{
				ChessService = cgc.Service;
				ChessService.ConnectToGame(out GameStateXML);
				this.LoadGameFromXMLString(GameStateXML);
				if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient)
					MyColor = ChessPieceColor.Black;
				else
					MyColor = ChessPieceColor.White;
				ConfigureUI();
				NetworkTimer.Enabled = true;
			}		
		}

		private void NetworkTimer_Tick(object sender, System.EventArgs e)
		{
			try
			{
				if (MyColor != chessboard.ColorToMove)
				{
					ChessPlayerType nextPlayer;
					if (chessboard.ColorToMove == ChessPieceColor.Black)
						nextPlayer = BlackPlayer;
					else
						nextPlayer = WhitePlayer;
					if (nextPlayer == ChessPlayerType.HumanIPNetworkClient)
					{
						if (EmilChessService.HasClientMove())
							Moving(EmilChessService.GetClientMove());
					
					} 
					else if (nextPlayer == ChessPlayerType.HumanIPNetworkServer)
					{
						if (ChessService.HasServerMove())
							Moving(ChessService.GetServerMove());
					}
				}
				ChessPlayerType myPlayer;
				if (MyColor == ChessPieceColor.Black)
					myPlayer = BlackPlayer;
				else
					myPlayer = WhitePlayer;
				if (myPlayer == ChessPlayerType.HumanIPNetworkClient)
				{
					if (ChessService.HasMsg4Client())
					{
						if (MessagesTB.Text != "")
							MessagesTB.Text = MessagesTB.Text + System.Environment.NewLine;
						MessagesTB.Text = MessagesTB.Text + ">" + ChessService.GetMsg4Client();
					}
				}
				else if (myPlayer == ChessPlayerType.HumanIPNetworkServer)
				{
					if (EmilChessService.HasMsg4Server())
					{
						if (MessagesTB.Text != "")
							MessagesTB.Text = MessagesTB.Text + System.Environment.NewLine;
						MessagesTB.Text = MessagesTB.Text + ">" + EmilChessService.GetMsg4Server();
					}
				}
			}
			catch (Exception ex)
			{
				chessGrid2.Enabled = false;
				NetworkTimer.Enabled = false;
				MessageBox.Show(this,Lost_connectionStr + System.Environment.NewLine + ex.Message);
			}

		}

		private void SendMsgTB_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if(e.KeyChar == (char)13)
			{
                e.Handled=true;
				ChessPlayerType myPlayer;
				if (MyColor == ChessPieceColor.Black)
					myPlayer = BlackPlayer;
				else
					myPlayer = WhitePlayer;
				if (myPlayer == ChessPlayerType.HumanIPNetworkClient)
				{
					if (MessagesTB.Text != "")
						MessagesTB.Text = MessagesTB.Text + System.Environment.NewLine;
                    MessagesTB.Text = MessagesTB.Text + "<" + SendMsgTB.Text;
					ChessService.SetMsg2Server(SendMsgTB.Text);
					SendMsgTB.Text = "";					
				}
				else if (myPlayer == ChessPlayerType.HumanIPNetworkServer)
				{
					if (MessagesTB.Text != "")
						MessagesTB.Text = MessagesTB.Text + System.Environment.NewLine;
					MessagesTB.Text = MessagesTB.Text + "<" + SendMsgTB.Text;
					EmilChessService.SetMsg2Client(SendMsgTB.Text);
					SendMsgTB.Text = null;
				}
			}
		}

		private void statusBar1_TextChanged(object sender, System.EventArgs e)
		{
			statusBar1.Refresh();
		}

		private void SendMsgTB_TextChanged(object sender, System.EventArgs e)
		{
		
		}


		private void SetColorAtBottom(ChessPieceColor c)
		{
			ColorAtBottom = c;
			if (ColorAtBottom == ChessPieceColor.Black)
			{
				ViewBlackAtBottomMI.Checked = true;
				ViewWhiteAtBottomMI.Checked = false;
				Captured2IG.DarkBackBrush = BlackBrush;
				Captured2IG.LightBackBrush = BlackBrush; 
				Captured1IG.DarkBackBrush = WhiteBrush;
				Captured1IG.LightBackBrush = WhiteBrush;
				for(int i=0; i<8; ++i)
					chessGrid2.SetYLabel(i,(i+1).ToString());
				for(int i=0; i<8; ++i)
					chessGrid2.SetXLabel(i,((char)('h'-i)).ToString());
			}
			else
			{
				ViewBlackAtBottomMI.Checked = false;
				ViewWhiteAtBottomMI.Checked = true;
				Captured1IG.DarkBackBrush = BlackBrush;
				Captured1IG.LightBackBrush = BlackBrush; 
				Captured2IG.DarkBackBrush = WhiteBrush;
				Captured2IG.LightBackBrush = WhiteBrush;
				for(int i=0; i<8; ++i)
					chessGrid2.SetYLabel(i,(8-i).ToString());
				for(int i=0; i<8; ++i)
					chessGrid2.SetXLabel(i,((char)('a'+i)).ToString());
			}
			
			this.DisplayChessBoard();
			chessGrid2.Refresh();
		}

		private void DecideColorAtBottom()
		{
			if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient
				|| BlackPlayer == ChessPlayerType.HumanIPNetworkServer
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkClient
				|| WhitePlayer == ChessPlayerType.HumanIPNetworkServer)
			{
				SetColorAtBottom(MyColor);
			}
			else if (BlackPlayer == ChessPlayerType.HumanEmail
				|| WhitePlayer == ChessPlayerType.HumanEmail)				
			{
				SetColorAtBottom(chessboard.ColorToMove);
			}
			else
			{
				if (BlackPlayer == ChessPlayerType.Human && WhitePlayer == ChessPlayerType.Human)
				{
					SetColorAtBottom(chessboard.ColorToMove);
				}
				else if (BlackPlayer == ChessPlayerType.Human)
				{
					SetColorAtBottom(ChessPieceColor.Black);
				}
				else
				{
					SetColorAtBottom(ChessPieceColor.White);
				}
			}
		}

		private void ViewBlackAtBottomMI_Click(object sender, System.EventArgs e)
		{
			SetColorAtBottom(ChessPieceColor.Black);
		
		}

		private void ViewWhiteAtBottomMI_Click(object sender, System.EventArgs e)
		{
			SetColorAtBottom(ChessPieceColor.Black);
		}

		private void FileNewGameViaEmailMI_Click(object sender, System.EventArgs e)
		{
			ChessAppNewGameViaEmail cge = new ChessAppNewGameViaEmail(rm,HelpFilePath);
			DialogResult dr;
			//string GameStateXML;
			cge.MyEmailAddress = this.EmailAddress;
			cge.SmtpServerAddress = this.SMTPServer;
			dr = cge.ShowDialog(this);
			if (dr == DialogResult.OK)
			{
				BlackPlayer = ChessPlayerType.HumanEmail;
				WhitePlayer = ChessPlayerType.HumanEmail;
				MyColor = cge.MyColor;
				if (MyColor == ChessPieceColor.Black)
				{
					BlackEmailAddress = cge.MyEmailAddress;
					WhiteEmailAddress = cge.OpponentEmailAddress;
				}
				else
				{
					BlackEmailAddress = cge.OpponentEmailAddress;
					WhiteEmailAddress = cge.MyEmailAddress;
				}
				GameName = cge.GameName;
				StartNewGame();
				ConfigureUI();
				if (MyColor == ChessPieceColor.Black)
				{
					SendGameByEmail(cge.GameName, cge.MyEmailAddress, cge.OpponentEmailAddress, cge.SmtpServerAddress);
				}
				else
				{
					// Disable UI
				}
			/*	ChessService = cgc.Service;
				ChessService.ConnectToGame(out GameStateXML);
				this.LoadGameFromXMLString(GameStateXML);
				if (BlackPlayer == ChessPlayerType.HumanIPNetworkClient)
					MyColor = ChessPieceColor.Black;
				else
					MyColor = ChessPieceColor.White;
				ConfigureUI();
				NetworkTimer.Enabled = true;*/
			}				
		}

		private bool SendGameByEmail(string GameName, string MyEmail, string OppEmail, string SMTPServer)
		{
			bool retVal=true;
			ChangedSinceSaved = false;
			string FileName=ProgramDir + @"\" + GameName + "_" + chessboard.NumberOfMovePlayed.ToString("000") + ".cgs";
			this.SaveGame(FileName);
			MailMessage mm = new MailMessage();
			mm.From = MyEmail;
			mm.To = OppEmail;
			mm.Subject = GameName + "_" + chessboard.NumberOfMovePlayed.ToString("000");
			mm.Body = "EmilChess game";
			MailAttachment ma = new MailAttachment(FileName);
			mm.Attachments.Add(ma);
			ChessAppSendMail csm = new ChessAppSendMail(rm,HelpFilePath,mm);
			csm.ShowDialog();
			string SaveServer;
			if (SMTPServer != null && SMTPServer != "")
			{
				SaveServer = SmtpMail.SmtpServer;
				SmtpMail.SmtpServer = SMTPServer;
				try 
				{
					SmtpMail.Send(mm);
					ChangedSinceSaved = true;
				}
				catch (System.Web.HttpException httpEx)
				{
					//Change_smtp_server_with
					MessageBox.Show(this,Invalid_smtp_serverStr + " \"" + SMTPServer + "\"" + System.Environment.NewLine + 
						Change_smtp_server_withStr + " \"" + rm.GetString("FileMI") + "->" + rm.GetString("FileOptionsMI") + "\"" + System.Environment.NewLine + 
						httpEx.Message,Sending_mail_failed_becauseStr);
					retVal = false;
				}
				catch (Exception ex)
				{
					MessageBox.Show(this,Invalid_smtp_serverStr + " \"" + SMTPServer + "\"" + System.Environment.NewLine + 
						Change_smtp_server_withStr + " \"" + rm.GetString("FileMI") + "->" + rm.GetString("FileOptionsMI") + "\"" + System.Environment.NewLine + 
						ex.Message,Sending_mail_failed_becauseStr);
					retVal = false;
				}				
				SmtpMail.SmtpServer = SaveServer;
			}
			else
			{
				MessageBox.Show(this,Invalid_smtp_serverStr + " \"" + SMTPServer + "\"" + System.Environment.NewLine + 
					Change_smtp_server_withStr + " \"" + rm.GetString("FileMI") + "->" + rm.GetString("FileOptionsMI") + "\"",
					Sending_mail_failed_becauseStr);
				retVal = false;
				/*try
				{
					SmtpMail.Send(mm);
				}
				catch (Exception ex)
				{
					MessageBox.Show(this,Invalid_smtp_serverStr + " \"" + SMTPServer + "\"" + System.Environment.NewLine + 
						Change_smtp_server_withStr + " \"" + rm.GetString("FileMI") + "->" + rm.GetString("FileOptionsMI") + "\"" + System.Environment.NewLine + 
						ex.Message,Sending_mail_failed_becauseStr);
					retVal = false;
				}*/
			}
			try
			{
				File.Delete(FileName);
			}
			catch (Exception)
			{
			}
			return retVal;
		}

		private void menuItem5_Click_1(object sender, System.EventArgs e)
		{
			this.ResizeWindow();
			//this.chessGrid2.Refresh();
		}

		private void ChessWindow_Activated(object sender, System.EventArgs e)
		{
			this.ResizeWindow();		
		}

		private void imageGrid1_Load(object sender, System.EventArgs e)
		{
		
		}

		private void HelpContentsMI_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,DataDir + @"\EmilChess_sv.chm", HelpNavigator.TableOfContents);
		}

		private void HelpIndexMI_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,DataDir + @"\EmilChess_sv.chm", HelpNavigator.Index);		
		}
		
	}
}
