using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Net;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Globalization;
using System.Resources;


namespace ChessApplication
{
	/// <summary>
	/// Summary description for ChessAppNewNetworkGame.
	/// </summary>
	public class ChessAppNewGameAsServer : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button CancelBtn;
		private System.Windows.Forms.TextBox PortTB;
		private System.Windows.Forms.TextBox IPAddressTB;
		private System.Windows.Forms.GroupBox MyColorGB;
		private System.Windows.Forms.RadioButton IAmWhiteRB;
		private System.Windows.Forms.RadioButton IAmBlackRB;
		private System.Windows.Forms.Button StartListenBtn;
		private TcpServerChannel channel;
		private int counter;
		private ChessWindow cw;
		private bool ContinuationOfGame;
		private System.Windows.Forms.ProgressBar progressBar1;
		private System.Windows.Forms.Timer timer1;
		private System.Windows.Forms.Label ListenPortLabel;
		private System.Windows.Forms.Label ListenIPAddressLabel;
		private System.Windows.Forms.Button HelpBtn;
		private System.ComponentModel.IContainer components;
		private string HelpFilePath;

		/// <summary>
		/// Constructor for new network game dialog
		/// </summary>
		/// <param name="rm"> Used to localize the dialog</param>
		/// <param name="cw"> The chesswindow that this dialog belong to</param>
		/// <param name="ContinuationOfGame"> Determine if My Color should be shown (shouldn't be shown for saved games)</param>
		/// <param name="HelpFilePath"> Used to find the help file</param>
		public ChessAppNewGameAsServer(ResourceManager rm, ChessWindow cw, bool ContinuationOfGame, string HelpFilePath)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.ContinuationOfGame = ContinuationOfGame;
			counter = 0;
			this.cw = cw;
			try 
			{
				string sHostName = Dns.GetHostName();
    
				IPHostEntry ipheThisHost = Dns.GetHostByName(sHostName);
				IPAddress ipThisAddr = ipheThisHost.AddressList[0];
  
				IPAddressTB.Text = ipThisAddr.ToString();
			}
			catch (Exception) 
			{
				IPAddressTB.Text = "127.0.0.1";
			}
			this.HelpFilePath = HelpFilePath;
			this.Text = rm.GetString("ChessAppNewGameAsServer");
			ListenIPAddressLabel.Text = rm.GetString("ListenIPAddressLabel");
			ListenPortLabel.Text = rm.GetString("ListenPortLabel");
			MyColorGB.Text = rm.GetString("MyColorGB");
			StartListenBtn.Text = rm.GetString("StartListenBtn");
			HelpBtn.Text = rm.GetString("HelpBtn");
			CancelBtn.Text = rm.GetString("CancelBtn");
			IAmBlackRB.Text = rm.GetString("Black");
			IAmWhiteRB.Text = rm.GetString("White");
			if (ContinuationOfGame)
				MyColorGB.Visible = false;
			else
				MyColorGB.Visible = true;

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// The port to listen to.
		/// </summary>
		public int Port
		{			
			get 
			{
				return System.Convert.ToInt32(PortTB.Text);
			}
			set
			{
				PortTB.Text = value.ToString();
			}
		}

		/// <summary>
		/// The color of the ChessServer.
		/// </summary>
		public ChessPieceColor MyColor
		{
			get 
			{
				if (IAmBlackRB.Checked)
                    return ChessPieceColor.Black;
				else
					return ChessPieceColor.White;
			}
			set
			{
				if (value == ChessPieceColor.Black)
					IAmBlackRB.Checked = true;
				else
					IAmWhiteRB.Checked = true;
			}
		}

		/// <summary>
		/// Determine if i am white player.
		/// </summary>
		/// <returns> Returns true if i am white player</returns>
		public bool IsWhite()
		{
			return IAmWhiteRB.Checked;
		}

		/// <summary>
		/// Determine if i am black player.
		/// </summary>
		/// <returns> Returns true if i am black player</returns>
		public bool IsBlack()
		{
			return IAmBlackRB.Checked;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.ListenPortLabel = new System.Windows.Forms.Label();
			this.CancelBtn = new System.Windows.Forms.Button();
			this.StartListenBtn = new System.Windows.Forms.Button();
			this.PortTB = new System.Windows.Forms.TextBox();
			this.IPAddressTB = new System.Windows.Forms.TextBox();
			this.ListenIPAddressLabel = new System.Windows.Forms.Label();
			this.MyColorGB = new System.Windows.Forms.GroupBox();
			this.IAmWhiteRB = new System.Windows.Forms.RadioButton();
			this.IAmBlackRB = new System.Windows.Forms.RadioButton();
			this.progressBar1 = new System.Windows.Forms.ProgressBar();
			this.timer1 = new System.Windows.Forms.Timer(this.components);
			this.HelpBtn = new System.Windows.Forms.Button();
			this.MyColorGB.SuspendLayout();
			this.SuspendLayout();
			// 
			// ListenPortLabel
			// 
			this.ListenPortLabel.Location = new System.Drawing.Point(16, 48);
			this.ListenPortLabel.Name = "ListenPortLabel";
			this.ListenPortLabel.Size = new System.Drawing.Size(96, 16);
			this.ListenPortLabel.TabIndex = 0;
			this.ListenPortLabel.Text = "Port to listen at:";
			// 
			// CancelBtn
			// 
			this.CancelBtn.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.CancelBtn.Location = new System.Drawing.Point(108, 168);
			this.CancelBtn.Name = "CancelBtn";
			this.CancelBtn.TabIndex = 6;
			this.CancelBtn.Text = "Cancel";
			// 
			// StartListenBtn
			// 
			this.StartListenBtn.Location = new System.Drawing.Point(8, 168);
			this.StartListenBtn.Name = "StartListenBtn";
			this.StartListenBtn.TabIndex = 5;
			this.StartListenBtn.Text = "Start Listen";
			this.StartListenBtn.Click += new System.EventHandler(this.StartListenBtn_Click);
			// 
			// PortTB
			// 
			this.PortTB.Location = new System.Drawing.Point(136, 48);
			this.PortTB.Name = "PortTB";
			this.PortTB.Size = new System.Drawing.Size(96, 20);
			this.PortTB.TabIndex = 7;
			this.PortTB.Text = "8001";
			// 
			// IPAddressTB
			// 
			this.IPAddressTB.Enabled = false;
			this.IPAddressTB.Location = new System.Drawing.Point(136, 24);
			this.IPAddressTB.Name = "IPAddressTB";
			this.IPAddressTB.Size = new System.Drawing.Size(96, 20);
			this.IPAddressTB.TabIndex = 9;
			this.IPAddressTB.Text = "127.0.0.1";
			// 
			// ListenIPAddressLabel
			// 
			this.ListenIPAddressLabel.Location = new System.Drawing.Point(16, 24);
			this.ListenIPAddressLabel.Name = "ListenIPAddressLabel";
			this.ListenIPAddressLabel.Size = new System.Drawing.Size(96, 16);
			this.ListenIPAddressLabel.TabIndex = 8;
			this.ListenIPAddressLabel.Text = "My IP Address:";
			// 
			// MyColorGB
			// 
			this.MyColorGB.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.IAmWhiteRB,
																					this.IAmBlackRB});
			this.MyColorGB.Location = new System.Drawing.Point(80, 80);
			this.MyColorGB.Name = "MyColorGB";
			this.MyColorGB.Size = new System.Drawing.Size(120, 80);
			this.MyColorGB.TabIndex = 10;
			this.MyColorGB.TabStop = false;
			this.MyColorGB.Text = "My Color";
			// 
			// IAmWhiteRB
			// 
			this.IAmWhiteRB.Checked = true;
			this.IAmWhiteRB.Location = new System.Drawing.Point(16, 48);
			this.IAmWhiteRB.Name = "IAmWhiteRB";
			this.IAmWhiteRB.Size = new System.Drawing.Size(96, 24);
			this.IAmWhiteRB.TabIndex = 1;
			this.IAmWhiteRB.TabStop = true;
			this.IAmWhiteRB.Text = "White";
			// 
			// IAmBlackRB
			// 
			this.IAmBlackRB.Location = new System.Drawing.Point(16, 24);
			this.IAmBlackRB.Name = "IAmBlackRB";
			this.IAmBlackRB.Size = new System.Drawing.Size(96, 16);
			this.IAmBlackRB.TabIndex = 0;
			this.IAmBlackRB.Text = "Black";
			// 
			// progressBar1
			// 
			this.progressBar1.Location = new System.Drawing.Point(0, 200);
			this.progressBar1.Name = "progressBar1";
			this.progressBar1.Size = new System.Drawing.Size(300, 24);
			this.progressBar1.TabIndex = 11;
			this.progressBar1.Visible = false;
			// 
			// timer1
			// 
			this.timer1.Tick += new System.EventHandler(this.timer1_Tick);
			// 
			// HelpBtn
			// 
			this.HelpBtn.Location = new System.Drawing.Point(208, 168);
			this.HelpBtn.Name = "HelpBtn";
			this.HelpBtn.TabIndex = 16;
			this.HelpBtn.Text = "Help";
			this.HelpBtn.Click += new System.EventHandler(this.HelpBtn_Click);
			// 
			// ChessAppNewGameAsServer
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(294, 224);
			this.ControlBox = false;
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.HelpBtn,
																		  this.progressBar1,
																		  this.MyColorGB,
																		  this.IPAddressTB,
																		  this.ListenIPAddressLabel,
																		  this.PortTB,
																		  this.CancelBtn,
																		  this.StartListenBtn,
																		  this.ListenPortLabel});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ChessAppNewGameAsServer";
			this.Text = "New Game As Server";
			this.Load += new System.EventHandler(this.ChessAppNewGameAsServer_Load);
			this.MyColorGB.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void ChessAppNewGameAsServer_Load(object sender, System.EventArgs e)
		{
		
		}

		private void StartListenBtn_Click(object sender, System.EventArgs e)
		{
			channel = new TcpServerChannel(this.Port);
			ChannelServices.RegisterChannel(channel);

			if (!ContinuationOfGame)
			{
				cw.MYColor = MyColor;
				cw.Port = this.Port;
				if (MyColor == ChessPieceColor.Black)
				{
					cw.BlackPlayerType = ChessPlayerType.HumanIPNetworkServer;
					cw.WhitePlayerType = ChessPlayerType.HumanIPNetworkClient;
				}
				else
				{
					cw.BlackPlayerType = ChessPlayerType.HumanIPNetworkClient;
					cw.WhitePlayerType = ChessPlayerType.HumanIPNetworkServer;
				}
				cw.StartNewGame();
			}

			cw.ConfigureUI();
   
			//Dropping the card in the box to register
			RemotingConfiguration.
				RegisterWellKnownServiceType(
				typeof(EmilChessService),//Name
				"EmilChessService", //What the outside world calls it
				WellKnownObjectMode.Singleton);//How to contact
			counter = 0;
			progressBar1.Visible = true;
			timer1.Start();
		}

		private void timer1_Tick(object sender, System.EventArgs e)
		{
			if (EmilChessService.HasClientConnected())
			{
				timer1.Stop();
				this.DialogResult = DialogResult.OK;
				progressBar1.Visible = false;
			}
			else
			{
				counter++;
				counter = counter % 100;
				progressBar1.Value = counter;
			}
		
		}

		private void HelpBtn_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,HelpFilePath,@"NewGameServer.htm");
		}
	}
}
