using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Globalization;
using System.Resources;

namespace ChessApplication
{
	/// <summary>
	/// Summary description for ChessAppNewGameDlg.
	/// </summary>
	public class ChessAppNewGameDlg : System.Windows.Forms.Form
	{
		private System.Windows.Forms.GroupBox WhitePlayerGB;
		private System.Windows.Forms.RadioButton WhiteComputerRB;
		private System.Windows.Forms.RadioButton WhiteHumanRB;
		private System.Windows.Forms.GroupBox BlackPlayerGB;
		private System.Windows.Forms.RadioButton BlackHumanRB;
		private System.Windows.Forms.RadioButton BlackComputerRB;
		private System.Windows.Forms.Button OKBtn;
		private System.Windows.Forms.Button CancelBtn;
		private System.Windows.Forms.NumericUpDown SearchDepthNUD;
		private System.Windows.Forms.ComboBox GenerationMethodCB;
		private System.Windows.Forms.NumericUpDown searchNodesNUD;
		private System.Windows.Forms.Label searchNodesLabel;
		private System.Windows.Forms.Label MoveGenMethodLabel;
		private System.Windows.Forms.Label SearchDephtLabel;
		private System.Windows.Forms.Button HelpBtn;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private string HelpFilePath;

		/// <summary>
		/// The constructor of the New Game dialog
		/// </summary>
		/// <param name="rm"> Used to localize the dialog</param>
		/// <param name="HelpFilePath"> Used to find the help file</param>
		public ChessAppNewGameDlg(ResourceManager rm, string HelpFilePath)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.HelpFilePath = HelpFilePath;
			this.Text = rm.GetString("ChessAppNewGameDlg");
		    WhitePlayerGB.Text = rm.GetString("WhitePlayerGB");
		    WhiteComputerRB.Text = rm.GetString("Computer");
		    WhiteHumanRB.Text = rm.GetString("Human");
		    BlackPlayerGB.Text = rm.GetString("BlackPlayerGB");
		    BlackHumanRB.Text = rm.GetString("Human");
		    BlackComputerRB.Text = rm.GetString("Computer");
		    OKBtn.Text = rm.GetString("OKBtn");
			CancelBtn.Text = rm.GetString("CancelBtn");
			HelpBtn.Text = rm.GetString("HelpBtn");
		    searchNodesLabel.Text = rm.GetString("searchNodesLabel");
		    MoveGenMethodLabel.Text = rm.GetString("MoveGenMethodLabel");
		    SearchDephtLabel.Text = rm.GetString("SearchDephtLabel");

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			GenerationMethodCB.SelectedIndex = 0;
		}

		/// <summary>
		/// Determine if white player is human (or computer).
		/// </summary>
		/// <returns> Returns true if white player is human</returns>
		public bool IsWhiteHuman()
		{
			return WhiteHumanRB.Checked;
		}

		/// <summary>
		/// Determine if white player is computer (or human).
		/// </summary>
		/// <returns> Returns true if white player is computer</returns>
		public bool IsWhiteComputer()
		{
			return WhiteComputerRB.Checked;
		}

		/// <summary>
		/// Determine if black player is human (or computer).
		/// </summary>
		/// <returns> Returns true if black player is human</returns>
		public bool IsBlackHuman()
		{
			return BlackHumanRB.Checked;
		}

		/// <summary>
		/// Determine if black player is computer (or human).
		/// </summary>
		/// <returns> Returns true if black player is the computer</returns>
		public bool IsBlackComputer()
		{
			return BlackComputerRB.Checked;
		}

		/// <summary>
		/// Get the search depth for the computer generated moves.
		/// </summary>
		public int GetSearchDepth()
		{
			return (int)SearchDepthNUD.Value;
		}

		/// <summary>
		/// Set the search depth for the computer generated moves.
		/// </summary>
		/// <param name="searchDepth"> The search depth</param>
		public void SetSearchDepth(int searchDepth)
		{
            SearchDepthNUD.Value = searchDepth;
		}

		/// <summary>
		/// Get the number of nodes that should be expanded during 
		/// the search for a move.
		/// </summary>
		public int GetSearchNodes()
		{
			return (int)searchNodesNUD.Value;
		}

		/// <summary>
		/// Set the number of nodes that should be expanded during 
		/// the search for a move.
		/// </summary>
		/// <param name="searchNodes"> The number of nodes per level to search</param>
		public void SetSearchNodes(int searchNodes)
		{
			searchNodesNUD.Value = searchNodes;
		}

		/// <summary>
		/// Get the move generation method i.e. the ChessEngine.
		/// </summary>
		/// <returns> ChessEngine string</returns>
		public string GetGenerationMethod()
		{
			return (string)GenerationMethodCB.SelectedItem;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.WhitePlayerGB = new System.Windows.Forms.GroupBox();
			this.WhiteHumanRB = new System.Windows.Forms.RadioButton();
			this.WhiteComputerRB = new System.Windows.Forms.RadioButton();
			this.BlackPlayerGB = new System.Windows.Forms.GroupBox();
			this.BlackHumanRB = new System.Windows.Forms.RadioButton();
			this.BlackComputerRB = new System.Windows.Forms.RadioButton();
			this.OKBtn = new System.Windows.Forms.Button();
			this.CancelBtn = new System.Windows.Forms.Button();
			this.SearchDepthNUD = new System.Windows.Forms.NumericUpDown();
			this.MoveGenMethodLabel = new System.Windows.Forms.Label();
			this.SearchDephtLabel = new System.Windows.Forms.Label();
			this.GenerationMethodCB = new System.Windows.Forms.ComboBox();
			this.searchNodesNUD = new System.Windows.Forms.NumericUpDown();
			this.searchNodesLabel = new System.Windows.Forms.Label();
			this.HelpBtn = new System.Windows.Forms.Button();
			this.WhitePlayerGB.SuspendLayout();
			this.BlackPlayerGB.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.SearchDepthNUD)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.searchNodesNUD)).BeginInit();
			this.SuspendLayout();
			// 
			// WhitePlayerGB
			// 
			this.WhitePlayerGB.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.WhiteHumanRB,
																						this.WhiteComputerRB});
			this.WhitePlayerGB.Location = new System.Drawing.Point(24, 16);
			this.WhitePlayerGB.Name = "WhitePlayerGB";
			this.WhitePlayerGB.Size = new System.Drawing.Size(120, 80);
			this.WhitePlayerGB.TabIndex = 1;
			this.WhitePlayerGB.TabStop = false;
			this.WhitePlayerGB.Text = "White player";
			// 
			// WhiteHumanRB
			// 
			this.WhiteHumanRB.Checked = true;
			this.WhiteHumanRB.Location = new System.Drawing.Point(16, 48);
			this.WhiteHumanRB.Name = "WhiteHumanRB";
			this.WhiteHumanRB.Size = new System.Drawing.Size(96, 24);
			this.WhiteHumanRB.TabIndex = 1;
			this.WhiteHumanRB.TabStop = true;
			this.WhiteHumanRB.Text = "Human";
			// 
			// WhiteComputerRB
			// 
			this.WhiteComputerRB.Location = new System.Drawing.Point(16, 24);
			this.WhiteComputerRB.Name = "WhiteComputerRB";
			this.WhiteComputerRB.Size = new System.Drawing.Size(96, 16);
			this.WhiteComputerRB.TabIndex = 0;
			this.WhiteComputerRB.Text = "Computer";
			this.WhiteComputerRB.CheckedChanged += new System.EventHandler(this.WhiteComputerRB_CheckedChanged);
			// 
			// BlackPlayerGB
			// 
			this.BlackPlayerGB.Controls.AddRange(new System.Windows.Forms.Control[] {
																						this.BlackHumanRB,
																						this.BlackComputerRB});
			this.BlackPlayerGB.Location = new System.Drawing.Point(176, 16);
			this.BlackPlayerGB.Name = "BlackPlayerGB";
			this.BlackPlayerGB.Size = new System.Drawing.Size(120, 80);
			this.BlackPlayerGB.TabIndex = 2;
			this.BlackPlayerGB.TabStop = false;
			this.BlackPlayerGB.Text = "Black player";
			// 
			// BlackHumanRB
			// 
			this.BlackHumanRB.Location = new System.Drawing.Point(16, 48);
			this.BlackHumanRB.Name = "BlackHumanRB";
			this.BlackHumanRB.Size = new System.Drawing.Size(96, 24);
			this.BlackHumanRB.TabIndex = 1;
			this.BlackHumanRB.Text = "Human";
			// 
			// BlackComputerRB
			// 
			this.BlackComputerRB.Checked = true;
			this.BlackComputerRB.Location = new System.Drawing.Point(16, 24);
			this.BlackComputerRB.Name = "BlackComputerRB";
			this.BlackComputerRB.Size = new System.Drawing.Size(96, 16);
			this.BlackComputerRB.TabIndex = 0;
			this.BlackComputerRB.TabStop = true;
			this.BlackComputerRB.Text = "Computer";
			this.BlackComputerRB.CheckedChanged += new System.EventHandler(this.BlackComputerRB_CheckedChanged);
			// 
			// OKBtn
			// 
			this.OKBtn.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.OKBtn.Location = new System.Drawing.Point(16, 192);
			this.OKBtn.Name = "OKBtn";
			this.OKBtn.TabIndex = 3;
			this.OKBtn.Text = "OK";
			// 
			// CancelBtn
			// 
			this.CancelBtn.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.CancelBtn.Location = new System.Drawing.Point(120, 192);
			this.CancelBtn.Name = "CancelBtn";
			this.CancelBtn.TabIndex = 4;
			this.CancelBtn.Text = "Cancel";
			// 
			// SearchDepthNUD
			// 
			this.SearchDepthNUD.Location = new System.Drawing.Point(216, 136);
			this.SearchDepthNUD.Minimum = new System.Decimal(new int[] {
																		   1,
																		   0,
																		   0,
																		   0});
			this.SearchDepthNUD.Name = "SearchDepthNUD";
			this.SearchDepthNUD.ReadOnly = true;
			this.SearchDepthNUD.Size = new System.Drawing.Size(48, 20);
			this.SearchDepthNUD.TabIndex = 5;
			this.SearchDepthNUD.Value = new System.Decimal(new int[] {
																		 4,
																		 0,
																		 0,
																		 0});
			// 
			// MoveGenMethodLabel
			// 
			this.MoveGenMethodLabel.Location = new System.Drawing.Point(24, 112);
			this.MoveGenMethodLabel.Name = "MoveGenMethodLabel";
			this.MoveGenMethodLabel.Size = new System.Drawing.Size(192, 24);
			this.MoveGenMethodLabel.TabIndex = 6;
			this.MoveGenMethodLabel.Text = "Move generation Search Depth:";
			// 
			// SearchDephtLabel
			// 
			this.SearchDephtLabel.Location = new System.Drawing.Point(24, 136);
			this.SearchDephtLabel.Name = "SearchDephtLabel";
			this.SearchDephtLabel.Size = new System.Drawing.Size(192, 16);
			this.SearchDephtLabel.TabIndex = 7;
			this.SearchDephtLabel.Text = "Move generation method:";
			// 
			// GenerationMethodCB
			// 
			this.GenerationMethodCB.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.GenerationMethodCB.Items.AddRange(new object[] {
																	"AlphaBeta",
																	"SearchNBest",
																	"Simple"});
			this.GenerationMethodCB.Location = new System.Drawing.Point(216, 112);
			this.GenerationMethodCB.Name = "GenerationMethodCB";
			this.GenerationMethodCB.Size = new System.Drawing.Size(88, 21);
			this.GenerationMethodCB.TabIndex = 8;
			this.GenerationMethodCB.SelectedIndexChanged += new System.EventHandler(this.GenerationMethodCB_SelectedIndexChanged);
			// 
			// searchNodesNUD
			// 
			this.searchNodesNUD.Location = new System.Drawing.Point(216, 160);
			this.searchNodesNUD.Minimum = new System.Decimal(new int[] {
																		   1,
																		   0,
																		   0,
																		   0});
			this.searchNodesNUD.Name = "searchNodesNUD";
			this.searchNodesNUD.ReadOnly = true;
			this.searchNodesNUD.Size = new System.Drawing.Size(48, 20);
			this.searchNodesNUD.TabIndex = 9;
			this.searchNodesNUD.Value = new System.Decimal(new int[] {
																		 8,
																		 0,
																		 0,
																		 0});
			this.searchNodesNUD.Visible = false;
			// 
			// searchNodesLabel
			// 
			this.searchNodesLabel.Location = new System.Drawing.Point(24, 160);
			this.searchNodesLabel.Name = "searchNodesLabel";
			this.searchNodesLabel.Size = new System.Drawing.Size(192, 24);
			this.searchNodesLabel.TabIndex = 10;
			this.searchNodesLabel.Text = "Number of best nodes to search:";
			this.searchNodesLabel.Visible = false;
			// 
			// HelpBtn
			// 
			this.HelpBtn.Location = new System.Drawing.Point(224, 192);
			this.HelpBtn.Name = "HelpBtn";
			this.HelpBtn.TabIndex = 11;
			this.HelpBtn.Text = "Help";
			this.HelpBtn.Click += new System.EventHandler(this.HelpBtn_Click);
			// 
			// ChessAppNewGameDlg
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(314, 232);
			this.ControlBox = false;
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.HelpBtn,
																		  this.searchNodesNUD,
																		  this.searchNodesLabel,
																		  this.SearchDepthNUD,
																		  this.GenerationMethodCB,
																		  this.SearchDephtLabel,
																		  this.MoveGenMethodLabel,
																		  this.CancelBtn,
																		  this.OKBtn,
																		  this.BlackPlayerGB,
																		  this.WhitePlayerGB});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ChessAppNewGameDlg";
			this.Text = "Start new game";
			this.Load += new System.EventHandler(this.ChessAppNewGameDlg_Load);
			this.WhitePlayerGB.ResumeLayout(false);
			this.BlackPlayerGB.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.SearchDepthNUD)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.searchNodesNUD)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void WhiteComputerRB_CheckedChanged(object sender, System.EventArgs e)
		{
			if (WhiteComputerRB.Checked && BlackComputerRB.Checked)
			{
				BlackHumanRB.Checked = true;
			}
		}

		private void BlackComputerRB_CheckedChanged(object sender, System.EventArgs e)
		{
			if (WhiteComputerRB.Checked && BlackComputerRB.Checked)
			{
				//WhiteComputerRB.Checked = false;
				WhiteHumanRB.Checked = true;
			}		
		}

		private void ChessAppNewGameDlg_Load(object sender, System.EventArgs e)
		{
		
		}

		private void GenerationMethodCB_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if ((string)GenerationMethodCB.SelectedItem == "SearchNBest")
			{
				searchNodesNUD.Visible = true;
				searchNodesLabel.Visible = true;
			}
			else
			{
				searchNodesNUD.Visible = false;
				searchNodesLabel.Visible = false;
			}
		
		}

		private void HelpBtn_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,HelpFilePath,@"NewGame.htm");
		}
	}
}
