using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Net;
using System.Globalization;
using System.Resources;


namespace ChessApplication
{
	/// <summary>
	/// Summary description for ChessAppNewNetworkGame.
	/// </summary>
	public class ChessAppNewGameViaEmail : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button CancelBtn;
		private System.Windows.Forms.GroupBox MyColorGB;
		private System.Windows.Forms.RadioButton IAmWhiteRB;
		private System.Windows.Forms.RadioButton IAmBlackRB;
		private System.Windows.Forms.ProgressBar progressBar1;
		private System.Windows.Forms.TextBox OpponentEmailAddressTB;
		private System.Windows.Forms.TextBox MyEmailAddressTB;
		private System.Windows.Forms.TextBox SmtpServerAddressTB;
		private System.Windows.Forms.Label SmtpServerAddressLabel;
		private System.Windows.Forms.Button OKBtn;
		private System.Windows.Forms.Label OpponentEmailAddressLabel;
		private System.Windows.Forms.Label MyEmailAddressLabel;
		private System.Windows.Forms.TextBox GameNameTB;
		private System.Windows.Forms.Label GameNameLabel;
		private System.Windows.Forms.Button HelpBtn;
		private System.ComponentModel.IContainer components = null;

		private string HelpFilePath;

		/// <summary>
		/// Constructor for new network game dialog
		/// </summary>
		/// <param name="rm"> Used to localize the dialog</param>
		/// <param name="HelpFilePath"> Used to find the help file</param>
		public ChessAppNewGameViaEmail(ResourceManager rm, string HelpFilePath)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			this.HelpFilePath = HelpFilePath;
			this.Text = rm.GetString("ChessAppNewGameViaEmail");
			GameNameLabel.Text = rm.GetString("GameNameLabel");
			MyEmailAddressLabel.Text = rm.GetString("MyEmailAddressLabel");
			OpponentEmailAddressLabel.Text = rm.GetString("OpponentEmailAddressLabel");
			SmtpServerAddressLabel.Text = rm.GetString("SmtpServerAddressLabel");
			OKBtn.Text = rm.GetString("OKBtn");
			CancelBtn.Text = rm.GetString("CancelBtn");
			HelpBtn.Text = rm.GetString("HelpBtn");
			MyColorGB.Text = rm.GetString("MyColorGB");
			IAmBlackRB.Text = rm.GetString("Black");
			IAmWhiteRB.Text = rm.GetString("White");

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// The name or title of this e-mail game.
		/// </summary>
		public string GameName
		{			
			get 
			{
				return GameNameTB.Text;
			}
			set
			{
				GameNameTB.Text = value;
			}
		}

		/// <summary>
		/// My e-mail address (will serve as the return address).
		/// </summary>
		public string MyEmailAddress
		{			
			get 
			{
				return MyEmailAddressTB.Text;
			}
			set
			{
				MyEmailAddressTB.Text = value;
			}
		}

		/// <summary>
		/// The opponents e-mail address (the game will be sent to it).
		/// </summary>
		public string OpponentEmailAddress
		{			
			get 
			{
				return OpponentEmailAddressTB.Text;
			}
			set
			{
				OpponentEmailAddressTB.Text = value;
			}
		}

		/// <summary>
		/// The address of the SMTP (Simple Mail Transfer Protocol) server the will be used fo sending the e-mail.
		/// </summary>
		public string SmtpServerAddress
		{			
			get 
			{
				return SmtpServerAddressTB.Text;
			}
			set
			{
				SmtpServerAddressTB.Text = value;
			}
		}

		/// <summary>
		/// The color of the ChessServer.
		/// </summary>
		public ChessPieceColor MyColor
		{
			get 
			{
				if (IAmBlackRB.Checked)
                    return ChessPieceColor.Black;
				else
					return ChessPieceColor.White;
			}
			set
			{
				if (value == ChessPieceColor.Black)
					IAmBlackRB.Checked = true;
				else
					IAmWhiteRB.Checked = true;
			}
		}

		/// <summary>
		/// Determine if i am white player.
		/// </summary>
		/// <returns> Returns true if i am white player</returns>
		public bool IsWhite()
		{
			return IAmWhiteRB.Checked;
		}

		/// <summary>
		/// Determine if i am black player.
		/// </summary>
		/// <returns> Returns true if i am black player</returns>
		public bool IsBlack()
		{
			return IAmBlackRB.Checked;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.OpponentEmailAddressLabel = new System.Windows.Forms.Label();
			this.CancelBtn = new System.Windows.Forms.Button();
			this.OKBtn = new System.Windows.Forms.Button();
			this.OpponentEmailAddressTB = new System.Windows.Forms.TextBox();
			this.MyEmailAddressTB = new System.Windows.Forms.TextBox();
			this.MyEmailAddressLabel = new System.Windows.Forms.Label();
			this.MyColorGB = new System.Windows.Forms.GroupBox();
			this.IAmWhiteRB = new System.Windows.Forms.RadioButton();
			this.IAmBlackRB = new System.Windows.Forms.RadioButton();
			this.progressBar1 = new System.Windows.Forms.ProgressBar();
			this.SmtpServerAddressTB = new System.Windows.Forms.TextBox();
			this.SmtpServerAddressLabel = new System.Windows.Forms.Label();
			this.GameNameTB = new System.Windows.Forms.TextBox();
			this.GameNameLabel = new System.Windows.Forms.Label();
			this.HelpBtn = new System.Windows.Forms.Button();
			this.MyColorGB.SuspendLayout();
			this.SuspendLayout();
			// 
			// OpponentEmailAddressLabel
			// 
			this.OpponentEmailAddressLabel.Location = new System.Drawing.Point(8, 56);
			this.OpponentEmailAddressLabel.Name = "OpponentEmailAddressLabel";
			this.OpponentEmailAddressLabel.Size = new System.Drawing.Size(144, 16);
			this.OpponentEmailAddressLabel.TabIndex = 0;
			this.OpponentEmailAddressLabel.Text = "Opponents e-mail address:";
			// 
			// CancelBtn
			// 
			this.CancelBtn.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.CancelBtn.Location = new System.Drawing.Point(116, 208);
			this.CancelBtn.Name = "CancelBtn";
			this.CancelBtn.TabIndex = 14;
			this.CancelBtn.Text = "Cancel";
			// 
			// OKBtn
			// 
			this.OKBtn.Location = new System.Drawing.Point(16, 208);
			this.OKBtn.Name = "OKBtn";
			this.OKBtn.TabIndex = 12;
			this.OKBtn.Text = "OK";
			this.OKBtn.Click += new System.EventHandler(this.StartListenBtn_Click);
			// 
			// OpponentEmailAddressTB
			// 
			this.OpponentEmailAddressTB.Location = new System.Drawing.Point(152, 56);
			this.OpponentEmailAddressTB.Name = "OpponentEmailAddressTB";
			this.OpponentEmailAddressTB.Size = new System.Drawing.Size(144, 20);
			this.OpponentEmailAddressTB.TabIndex = 4;
			this.OpponentEmailAddressTB.Text = "";
			// 
			// MyEmailAddressTB
			// 
			this.MyEmailAddressTB.Location = new System.Drawing.Point(152, 32);
			this.MyEmailAddressTB.Name = "MyEmailAddressTB";
			this.MyEmailAddressTB.Size = new System.Drawing.Size(144, 20);
			this.MyEmailAddressTB.TabIndex = 2;
			this.MyEmailAddressTB.Text = "";
			// 
			// MyEmailAddressLabel
			// 
			this.MyEmailAddressLabel.Location = new System.Drawing.Point(8, 32);
			this.MyEmailAddressLabel.Name = "MyEmailAddressLabel";
			this.MyEmailAddressLabel.Size = new System.Drawing.Size(144, 16);
			this.MyEmailAddressLabel.TabIndex = 8;
			this.MyEmailAddressLabel.Text = "My e-mail Address:";
			// 
			// MyColorGB
			// 
			this.MyColorGB.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.IAmWhiteRB,
																					this.IAmBlackRB});
			this.MyColorGB.Location = new System.Drawing.Point(80, 80);
			this.MyColorGB.Name = "MyColorGB";
			this.MyColorGB.Size = new System.Drawing.Size(120, 80);
			this.MyColorGB.TabIndex = 10;
			this.MyColorGB.TabStop = false;
			this.MyColorGB.Text = "My Color";
			// 
			// IAmWhiteRB
			// 
			this.IAmWhiteRB.Checked = true;
			this.IAmWhiteRB.Location = new System.Drawing.Point(16, 48);
			this.IAmWhiteRB.Name = "IAmWhiteRB";
			this.IAmWhiteRB.Size = new System.Drawing.Size(96, 24);
			this.IAmWhiteRB.TabIndex = 8;
			this.IAmWhiteRB.TabStop = true;
			this.IAmWhiteRB.Text = "White";
			// 
			// IAmBlackRB
			// 
			this.IAmBlackRB.Location = new System.Drawing.Point(16, 24);
			this.IAmBlackRB.Name = "IAmBlackRB";
			this.IAmBlackRB.Size = new System.Drawing.Size(96, 16);
			this.IAmBlackRB.TabIndex = 6;
			this.IAmBlackRB.Text = "Black";
			// 
			// progressBar1
			// 
			this.progressBar1.Location = new System.Drawing.Point(0, 240);
			this.progressBar1.Name = "progressBar1";
			this.progressBar1.Size = new System.Drawing.Size(312, 24);
			this.progressBar1.TabIndex = 11;
			this.progressBar1.Visible = false;
			// 
			// SmtpServerAddressTB
			// 
			this.SmtpServerAddressTB.Location = new System.Drawing.Point(152, 176);
			this.SmtpServerAddressTB.Name = "SmtpServerAddressTB";
			this.SmtpServerAddressTB.Size = new System.Drawing.Size(144, 20);
			this.SmtpServerAddressTB.TabIndex = 10;
			this.SmtpServerAddressTB.Text = "";
			this.SmtpServerAddressTB.Visible = false;
			// 
			// SmtpServerAddressLabel
			// 
			this.SmtpServerAddressLabel.Location = new System.Drawing.Point(8, 176);
			this.SmtpServerAddressLabel.Name = "SmtpServerAddressLabel";
			this.SmtpServerAddressLabel.Size = new System.Drawing.Size(144, 16);
			this.SmtpServerAddressLabel.TabIndex = 12;
			this.SmtpServerAddressLabel.Text = "SMTP Server address:";
			this.SmtpServerAddressLabel.Visible = false;
			// 
			// GameNameTB
			// 
			this.GameNameTB.Location = new System.Drawing.Point(152, 8);
			this.GameNameTB.Name = "GameNameTB";
			this.GameNameTB.Size = new System.Drawing.Size(144, 20);
			this.GameNameTB.TabIndex = 0;
			this.GameNameTB.Text = "";
			// 
			// GameNameLabel
			// 
			this.GameNameLabel.Location = new System.Drawing.Point(8, 8);
			this.GameNameLabel.Name = "GameNameLabel";
			this.GameNameLabel.Size = new System.Drawing.Size(144, 16);
			this.GameNameLabel.TabIndex = 14;
			this.GameNameLabel.Text = "Game name:";
			// 
			// HelpBtn
			// 
			this.HelpBtn.Location = new System.Drawing.Point(216, 208);
			this.HelpBtn.Name = "HelpBtn";
			this.HelpBtn.TabIndex = 15;
			this.HelpBtn.Text = "Help";
			this.HelpBtn.Click += new System.EventHandler(this.HelpBtn_Click);
			// 
			// ChessAppNewGameViaEmail
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(306, 264);
			this.ControlBox = false;
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.HelpBtn,
																		  this.GameNameTB,
																		  this.GameNameLabel,
																		  this.SmtpServerAddressTB,
																		  this.SmtpServerAddressLabel,
																		  this.progressBar1,
																		  this.MyColorGB,
																		  this.MyEmailAddressTB,
																		  this.MyEmailAddressLabel,
																		  this.OpponentEmailAddressTB,
																		  this.CancelBtn,
																		  this.OKBtn,
																		  this.OpponentEmailAddressLabel});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ChessAppNewGameViaEmail";
			this.Text = "New Game via e-mail";
			this.Load += new System.EventHandler(this.ChessAppNewGameAsServer_Load);
			this.MyColorGB.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		private void ChessAppNewGameAsServer_Load(object sender, System.EventArgs e)
		{
		
		}

		private void StartListenBtn_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.OK;
		}

		private void HelpBtn_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,HelpFilePath,@"NewGameEmail.htm");
		}
	}
}
