using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Globalization;
using System.Resources;

namespace ChessApplication
{
	/// <summary>
	/// Summary description for ChessAppOptions.
	/// </summary>
	public class ChessAppOptions : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label ValuePawnLabel;
		private System.Windows.Forms.Label ValueKnightLabel;
		private System.Windows.Forms.Label ValueBishopLabel;
		private System.Windows.Forms.Label ValueRookLabel;
		private System.Windows.Forms.Label ValueQueenLabel;
		private System.Windows.Forms.Label SkinLabel;
		private System.Windows.Forms.Button OKBtn;
		private System.Windows.Forms.Button CancelBtn;
		private System.Windows.Forms.Label HighLightTimeLabel;
		private System.Windows.Forms.TextBox ValuePawnTB;
		private System.Windows.Forms.TextBox ValueKnightTB;
		private System.Windows.Forms.TextBox ValueBishopTB;
		private System.Windows.Forms.TextBox ValueRookTB;
		private System.Windows.Forms.TextBox ValueQueenTB;
		private System.Windows.Forms.TextBox HighLightTimeTB;
		private System.Windows.Forms.Button BrowseBtn;
		private CultureInfo culture;
		private NumberFormatInfo numberFormat;
		private System.Windows.Forms.TextBox SkinTB;
		private System.Windows.Forms.OpenFileDialog SkinOpenFileDialog;
		private System.Windows.Forms.TextBox ServicePortTB;
		private System.Windows.Forms.Label ServicePortLabel;
		private System.Windows.Forms.TextBox EmailAddressTB;
		private System.Windows.Forms.Label EmailAddressLabel;
		private System.Windows.Forms.TextBox SMTPServerTB;
		private System.Windows.Forms.Label SMTPServerLabel;
		private System.Windows.Forms.Button HelpBtn;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.ComboBox LanguageCB;
		private System.Windows.Forms.Label LanguageLabel;
		private string HelpFilePath;

		/// <summary>
		/// Constructor of the ChessAppOptions dialog
		/// </summary>
		/// <param name="rm"> Used to localize the dialog</param>
		/// <param name="HelpFilePath"> Used to find the help file</param>
		public ChessAppOptions(ResourceManager rm, string HelpFilePath)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.HelpFilePath = HelpFilePath;

			this.Text = rm.GetString("ChessAppOptions");

			string Value_of_a = rm.GetString("Value_of_a");

			ValuePawnLabel.Text = Value_of_a + " " + rm.GetString("Pawn") + ":";
			ValueKnightLabel.Text = Value_of_a + " " + rm.GetString("Knight") + ":";
			ValueBishopLabel.Text = Value_of_a + " " + rm.GetString("Bishop") + ":";
			ValueRookLabel.Text = Value_of_a + " " + rm.GetString("Rook") + ":";
			ValueQueenLabel.Text = Value_of_a + " " + rm.GetString("Queen") + ":";
			HighLightTimeLabel.Text = rm.GetString("HighLightTimeLabel");
			ServicePortLabel.Text = rm.GetString("ServicePortLabel");
			EmailAddressLabel.Text = rm.GetString("EmailAddressLabel");
			SMTPServerLabel.Text = rm.GetString("SMTPServerLabel");

			SkinLabel.Text = rm.GetString("SkinLabel");
			OKBtn.Text = rm.GetString("OKBtn");
			CancelBtn.Text = rm.GetString("CancelBtn");
			BrowseBtn.Text = rm.GetString("BrowseBtn");
			HelpBtn.Text = rm.GetString("HelpBtn");

			LanguageLabel.Text = rm.GetString("LanguageLabel");

			SkinOpenFileDialog.Title = rm.GetString("SkinOpenFileDialog");
			SkinOpenFileDialog.Filter = rm.GetString("SkinOpenFileDialog.Filter");

			culture = CultureInfo.CurrentUICulture;

			numberFormat = culture.NumberFormat;

			this.LanguageCB.Items.Clear();

			foreach (CultureInfo ci in
				CultureInfo.GetCultures(CultureTypes.AllCultures))
			{
				this.LanguageCB.Items.Add(ci.Name);
			}
			LanguageCB.SelectedItem = CultureInfo.CurrentUICulture.Name;


			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// The value of a Bishop
		/// </summary>
		public double BishopValue 
		{
			get 
			{
				return System.Convert.ToDouble(ValueBishopTB.Text, numberFormat);
			}
			set
			{
				ValueBishopTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The value of a Knight
		/// </summary>
		public double KnightValue 
		{
			get 
			{
				return System.Convert.ToDouble(ValueKnightTB.Text, numberFormat);
			}
			set
			{
				ValueKnightTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The value of a Pawn
		/// </summary>
		public double PawnValue 
		{
			get 
			{
				return System.Convert.ToDouble(ValuePawnTB.Text, numberFormat);
			}
			set
			{
				ValuePawnTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The value of a Rook
		/// </summary>
		public double RookValue 
		{
			get 
			{
				return System.Convert.ToDouble(ValueRookTB.Text, numberFormat);
			}
			set
			{
				ValueRookTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The value of a Queen
		/// </summary>
		public double QueenValue 
		{
			get 
			{
				return System.Convert.ToDouble(ValueQueenTB.Text, numberFormat);
			}
			set
			{
				ValueQueenTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The number of seconds a move should be highlighted.
		/// </summary>
		public int MoveHightlightSec
		{
			get 
			{
				return System.Convert.ToInt32(HighLightTimeTB.Text, numberFormat);
			}
			set
			{
				HighLightTimeTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The port to listen to when hosting a game.
		/// </summary>
		public int ServicePort
		{
			get 
			{
				return System.Convert.ToInt32(ServicePortTB.Text, numberFormat);
			}
			set
			{
				ServicePortTB.Text = value.ToString(numberFormat);
			}
		}

		/// <summary>
		/// The e-mail address used as my e-mail address when starting a e-mail game.
		/// </summary>
		public string EmailAddress
		{
			get 
			{
				return EmailAddressTB.Text;
			}
			set
			{
				EmailAddressTB.Text = value;
			}
		}

		/// <summary>
		/// The SMTP server address used to send e-mails when starting a e-mail game.
		/// </summary>
		public string SMTPServer
		{
			get 
			{
				return SMTPServerTB.Text;
			}
			set
			{
				SMTPServerTB.Text = value;
			}
		}

		/// <summary>
		/// The path to the skin.
		/// </summary>
		public string SkinPath
		{
			get 
			{
				return SkinTB.Text;
			}
			set
			{
				SkinTB.Text = value;
			}
		}

		/// <summary>
		/// The Language code used for the UI.
		/// </summary>
		public string LanguageCode
		{
			get 
			{
				return (string)LanguageCB.SelectedItem;
			}
			set
			{
				LanguageCB.SelectedItem = value;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.ValuePawnLabel = new System.Windows.Forms.Label();
			this.ValueKnightLabel = new System.Windows.Forms.Label();
			this.ValueBishopLabel = new System.Windows.Forms.Label();
			this.ValueRookLabel = new System.Windows.Forms.Label();
			this.ValueQueenLabel = new System.Windows.Forms.Label();
			this.HighLightTimeLabel = new System.Windows.Forms.Label();
			this.ValuePawnTB = new System.Windows.Forms.TextBox();
			this.ValueKnightTB = new System.Windows.Forms.TextBox();
			this.ValueBishopTB = new System.Windows.Forms.TextBox();
			this.ValueRookTB = new System.Windows.Forms.TextBox();
			this.ValueQueenTB = new System.Windows.Forms.TextBox();
			this.HighLightTimeTB = new System.Windows.Forms.TextBox();
			this.SkinLabel = new System.Windows.Forms.Label();
			this.SkinTB = new System.Windows.Forms.TextBox();
			this.BrowseBtn = new System.Windows.Forms.Button();
			this.OKBtn = new System.Windows.Forms.Button();
			this.CancelBtn = new System.Windows.Forms.Button();
			this.SkinOpenFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.ServicePortTB = new System.Windows.Forms.TextBox();
			this.ServicePortLabel = new System.Windows.Forms.Label();
			this.EmailAddressTB = new System.Windows.Forms.TextBox();
			this.EmailAddressLabel = new System.Windows.Forms.Label();
			this.SMTPServerTB = new System.Windows.Forms.TextBox();
			this.SMTPServerLabel = new System.Windows.Forms.Label();
			this.HelpBtn = new System.Windows.Forms.Button();
			this.LanguageCB = new System.Windows.Forms.ComboBox();
			this.LanguageLabel = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// ValuePawnLabel
			// 
			this.ValuePawnLabel.Location = new System.Drawing.Point(16, 24);
			this.ValuePawnLabel.Name = "ValuePawnLabel";
			this.ValuePawnLabel.Size = new System.Drawing.Size(176, 16);
			this.ValuePawnLabel.TabIndex = 0;
			this.ValuePawnLabel.Text = "Value of a Pawn:";
			// 
			// ValueKnightLabel
			// 
			this.ValueKnightLabel.Location = new System.Drawing.Point(16, 48);
			this.ValueKnightLabel.Name = "ValueKnightLabel";
			this.ValueKnightLabel.Size = new System.Drawing.Size(176, 16);
			this.ValueKnightLabel.TabIndex = 1;
			this.ValueKnightLabel.Text = "Value of a Knight:";
			// 
			// ValueBishopLabel
			// 
			this.ValueBishopLabel.Location = new System.Drawing.Point(16, 72);
			this.ValueBishopLabel.Name = "ValueBishopLabel";
			this.ValueBishopLabel.Size = new System.Drawing.Size(176, 16);
			this.ValueBishopLabel.TabIndex = 2;
			this.ValueBishopLabel.Text = "Value of a Bishop:";
			// 
			// ValueRookLabel
			// 
			this.ValueRookLabel.Location = new System.Drawing.Point(16, 96);
			this.ValueRookLabel.Name = "ValueRookLabel";
			this.ValueRookLabel.Size = new System.Drawing.Size(176, 16);
			this.ValueRookLabel.TabIndex = 3;
			this.ValueRookLabel.Text = "Value of a Rook:";
			// 
			// ValueQueenLabel
			// 
			this.ValueQueenLabel.Location = new System.Drawing.Point(16, 120);
			this.ValueQueenLabel.Name = "ValueQueenLabel";
			this.ValueQueenLabel.Size = new System.Drawing.Size(176, 16);
			this.ValueQueenLabel.TabIndex = 4;
			this.ValueQueenLabel.Text = "Value of a Queen:";
			// 
			// HighLightTimeLabel
			// 
			this.HighLightTimeLabel.Location = new System.Drawing.Point(16, 152);
			this.HighLightTimeLabel.Name = "HighLightTimeLabel";
			this.HighLightTimeLabel.Size = new System.Drawing.Size(184, 16);
			this.HighLightTimeLabel.TabIndex = 5;
			this.HighLightTimeLabel.Text = "Move highlight time (seconds):";
			// 
			// ValuePawnTB
			// 
			this.ValuePawnTB.Location = new System.Drawing.Point(200, 24);
			this.ValuePawnTB.Name = "ValuePawnTB";
			this.ValuePawnTB.TabIndex = 6;
			this.ValuePawnTB.Text = "1";
			// 
			// ValueKnightTB
			// 
			this.ValueKnightTB.Location = new System.Drawing.Point(200, 48);
			this.ValueKnightTB.Name = "ValueKnightTB";
			this.ValueKnightTB.TabIndex = 7;
			this.ValueKnightTB.Text = "3";
			// 
			// ValueBishopTB
			// 
			this.ValueBishopTB.Location = new System.Drawing.Point(200, 72);
			this.ValueBishopTB.Name = "ValueBishopTB";
			this.ValueBishopTB.TabIndex = 8;
			this.ValueBishopTB.Text = "3";
			// 
			// ValueRookTB
			// 
			this.ValueRookTB.Location = new System.Drawing.Point(200, 96);
			this.ValueRookTB.Name = "ValueRookTB";
			this.ValueRookTB.TabIndex = 9;
			this.ValueRookTB.Text = "5";
			// 
			// ValueQueenTB
			// 
			this.ValueQueenTB.Location = new System.Drawing.Point(200, 120);
			this.ValueQueenTB.Name = "ValueQueenTB";
			this.ValueQueenTB.TabIndex = 10;
			this.ValueQueenTB.Text = "8";
			// 
			// HighLightTimeTB
			// 
			this.HighLightTimeTB.Location = new System.Drawing.Point(200, 152);
			this.HighLightTimeTB.Name = "HighLightTimeTB";
			this.HighLightTimeTB.TabIndex = 11;
			this.HighLightTimeTB.Text = "3";
			// 
			// SkinLabel
			// 
			this.SkinLabel.Location = new System.Drawing.Point(16, 304);
			this.SkinLabel.Name = "SkinLabel";
			this.SkinLabel.Size = new System.Drawing.Size(32, 16);
			this.SkinLabel.TabIndex = 12;
			this.SkinLabel.Text = "Skin:";
			// 
			// SkinTB
			// 
			this.SkinTB.Location = new System.Drawing.Point(56, 304);
			this.SkinTB.Name = "SkinTB";
			this.SkinTB.Size = new System.Drawing.Size(176, 20);
			this.SkinTB.TabIndex = 13;
			this.SkinTB.Text = "Skins\\Standard.xml";
			// 
			// BrowseBtn
			// 
			this.BrowseBtn.Location = new System.Drawing.Point(240, 304);
			this.BrowseBtn.Name = "BrowseBtn";
			this.BrowseBtn.Size = new System.Drawing.Size(64, 24);
			this.BrowseBtn.TabIndex = 14;
			this.BrowseBtn.Text = "Browse";
			this.BrowseBtn.Click += new System.EventHandler(this.BrowseBtn_Click);
			// 
			// OKBtn
			// 
			this.OKBtn.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.OKBtn.Location = new System.Drawing.Point(16, 344);
			this.OKBtn.Name = "OKBtn";
			this.OKBtn.Size = new System.Drawing.Size(64, 24);
			this.OKBtn.TabIndex = 15;
			this.OKBtn.Text = "OK";
			this.OKBtn.Click += new System.EventHandler(this.OKBtn_Click);
			// 
			// CancelBtn
			// 
			this.CancelBtn.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.CancelBtn.Location = new System.Drawing.Point(120, 344);
			this.CancelBtn.Name = "CancelBtn";
			this.CancelBtn.Size = new System.Drawing.Size(64, 24);
			this.CancelBtn.TabIndex = 16;
			this.CancelBtn.Text = "Cancel";
			this.CancelBtn.Click += new System.EventHandler(this.CancelBtn_Click);
			// 
			// SkinOpenFileDialog
			// 
			this.SkinOpenFileDialog.DefaultExt = "xml";
			this.SkinOpenFileDialog.Filter = "Skin files in XML format (*.xml)|*.xml";
			this.SkinOpenFileDialog.Title = "Open Skin files";
			// 
			// ServicePortTB
			// 
			this.ServicePortTB.Location = new System.Drawing.Point(203, 184);
			this.ServicePortTB.Name = "ServicePortTB";
			this.ServicePortTB.TabIndex = 18;
			this.ServicePortTB.Text = "8001";
			// 
			// ServicePortLabel
			// 
			this.ServicePortLabel.Location = new System.Drawing.Point(19, 184);
			this.ServicePortLabel.Name = "ServicePortLabel";
			this.ServicePortLabel.Size = new System.Drawing.Size(176, 16);
			this.ServicePortLabel.TabIndex = 17;
			this.ServicePortLabel.Text = "Port to listen at:";
			// 
			// EmailAddressTB
			// 
			this.EmailAddressTB.Location = new System.Drawing.Point(203, 208);
			this.EmailAddressTB.Name = "EmailAddressTB";
			this.EmailAddressTB.TabIndex = 20;
			this.EmailAddressTB.Text = "";
			this.EmailAddressTB.TextChanged += new System.EventHandler(this.textBox2_TextChanged);
			// 
			// EmailAddressLabel
			// 
			this.EmailAddressLabel.Location = new System.Drawing.Point(19, 208);
			this.EmailAddressLabel.Name = "EmailAddressLabel";
			this.EmailAddressLabel.Size = new System.Drawing.Size(176, 16);
			this.EmailAddressLabel.TabIndex = 19;
			this.EmailAddressLabel.Text = "My e-mail address:";
			this.EmailAddressLabel.Click += new System.EventHandler(this.label2_Click);
			// 
			// SMTPServerTB
			// 
			this.SMTPServerTB.Location = new System.Drawing.Point(203, 232);
			this.SMTPServerTB.Name = "SMTPServerTB";
			this.SMTPServerTB.TabIndex = 22;
			this.SMTPServerTB.Text = "";
			// 
			// SMTPServerLabel
			// 
			this.SMTPServerLabel.Location = new System.Drawing.Point(19, 232);
			this.SMTPServerLabel.Name = "SMTPServerLabel";
			this.SMTPServerLabel.Size = new System.Drawing.Size(176, 16);
			this.SMTPServerLabel.TabIndex = 21;
			this.SMTPServerLabel.Text = "SMTP Server to use:";
			// 
			// HelpBtn
			// 
			this.HelpBtn.Location = new System.Drawing.Point(232, 344);
			this.HelpBtn.Name = "HelpBtn";
			this.HelpBtn.TabIndex = 23;
			this.HelpBtn.Text = "Help";
			this.HelpBtn.Click += new System.EventHandler(this.HelpBtn_Click);
			// 
			// LanguageCB
			// 
			this.LanguageCB.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.LanguageCB.Location = new System.Drawing.Point(200, 264);
			this.LanguageCB.Name = "LanguageCB";
			this.LanguageCB.Size = new System.Drawing.Size(104, 21);
			this.LanguageCB.TabIndex = 24;
			// 
			// LanguageLabel
			// 
			this.LanguageLabel.Location = new System.Drawing.Point(19, 264);
			this.LanguageLabel.Name = "LanguageLabel";
			this.LanguageLabel.Size = new System.Drawing.Size(168, 16);
			this.LanguageLabel.TabIndex = 25;
			this.LanguageLabel.Text = "Language code:";
			// 
			// ChessAppOptions
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(322, 376);
			this.ControlBox = false;
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.LanguageLabel,
																		  this.LanguageCB,
																		  this.HelpBtn,
																		  this.SMTPServerTB,
																		  this.SMTPServerLabel,
																		  this.EmailAddressTB,
																		  this.EmailAddressLabel,
																		  this.ServicePortTB,
																		  this.ServicePortLabel,
																		  this.CancelBtn,
																		  this.OKBtn,
																		  this.BrowseBtn,
																		  this.SkinTB,
																		  this.SkinLabel,
																		  this.HighLightTimeTB,
																		  this.ValueQueenTB,
																		  this.ValueRookTB,
																		  this.ValueBishopTB,
																		  this.ValueKnightTB,
																		  this.ValuePawnTB,
																		  this.HighLightTimeLabel,
																		  this.ValueQueenLabel,
																		  this.ValueRookLabel,
																		  this.ValueBishopLabel,
																		  this.ValueKnightLabel,
																		  this.ValuePawnLabel});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "ChessAppOptions";
			this.Text = "Options";
			this.ResumeLayout(false);

		}
		#endregion

		private void BrowseBtn_Click(object sender, System.EventArgs e)
		{
			if (DialogResult.OK == SkinOpenFileDialog.ShowDialog(this))
			{
				SkinTB.Text = SkinOpenFileDialog.FileName;
			}
		}

		private void label2_Click(object sender, System.EventArgs e)
		{
		
		}

		private void textBox2_TextChanged(object sender, System.EventArgs e)
		{
		
		}

		private void CancelBtn_Click(object sender, System.EventArgs e)
		{
		
		}

		private void OKBtn_Click(object sender, System.EventArgs e)
		{
		
		}

		private void HelpBtn_Click(object sender, System.EventArgs e)
		{
			Help.ShowHelp(this,HelpFilePath,@"Options.htm");
		}
	}
}
