using System;

namespace ChessApplication
{
	/// <summary>
	/// Summary description for EmilChessService.
	/// </summary>
	public class EmilChessService : MarshalByRefObject 
	{

		private static ChessWindow chesswindow=null;
		private static bool ClientConnected=false;
		private static ChessMove ClientMove=null;
		private static ChessMove ServerMove=null;
		private static string Msg2Client=null;
		private static string Msg2Server=null;
		/// <summary>
		/// The constructor of the service
		/// </summary>
		public EmilChessService()
		{
			//
			// TODO: Add constructor logic here
			//
		}
		/// <summary>
		/// Connects to the game.
		/// </summary>
		/// <param name="GameStateXML"> The game state in XML format</param>
		public void ConnectToGame(out string GameStateXML)
		{
			ClientConnected = true;
			GameStateXML = chesswindow.SaveGameToXMLString();
		}
		/// <summary>
		/// Set the ChessWindow that will be used by the Service.
		/// </summary>
		/// <param name="cw">The ChessWindow that will be used by server.</param>
		public static void SetChessWindow(ChessWindow cw)
		{
			chesswindow = cw;
		}
		/// <summary>
		/// Determine if a client has connected.
		/// </summary>
		/// <returns> Returns true if a client has connected</returns>
		public static bool HasClientConnected()
		{
			return ClientConnected;
		}
		/// <summary>
		/// Determine if a client has posted a move.
		/// </summary>
		/// <returns> Returns true if a client has posted a move</returns>
		public static bool HasClientMove()
		{
			return ClientMove != null;
		}
		/// <summary>
		/// Get the move the client has posted.
		/// </summary>
		/// <returns> Returns the move the client has posted</returns>
		public static ChessMove GetClientMove()
		{
			lock (typeof(EmilChessService))
			{
				ChessMove temp=ClientMove;
				ClientMove = null;
				return temp;
			}			
		}
		/// <summary>
		/// Set the move of the client.
		/// </summary>
		/// <param name="move"> The move of the client</param>
		public void SetClientMove(ChessMove move)
		{
			lock (typeof(EmilChessService))
			{
				ClientMove = move;
			}
		}
		/// <summary>
		/// Determine if the server has posted a move.
		/// </summary>
		/// <returns> Returns true if the server has posted a move</returns>
		public bool HasServerMove()
		{
			return ServerMove != null;
		}
		/// <summary>
		/// Get the move the server has posted.
		/// </summary>
		/// <returns> Returns the move the server has posted</returns>
		public ChessMove GetServerMove()
		{
			lock (typeof(EmilChessService))
			{
				ChessMove temp=ServerMove;
				ServerMove = null;
				return temp;
			}
		}
		/// <summary>
		/// Set the move of the server.
		/// </summary>
		/// <param name="move"> The move of the server</param>
		public static void SetServerMove(ChessMove move)
		{
			lock (typeof(EmilChessService))
			{
				ServerMove = move;
			}
		}
		/// <summary>
		/// Determine if there are a message to the client.
		/// </summary>
		/// <returns> Returns true if there are any messages to the client</returns>
		public bool HasMsg4Client()
		{
			return Msg2Client != null;
		}
		/// <summary>
		/// Get the messages to the client.
		/// </summary>
		/// <returns> Returns the message to the client</returns>
		public string GetMsg4Client()
		{
			lock (typeof(EmilChessService))
			{
				string temp=Msg2Client;
				Msg2Client = null;
				return temp;
			}
		}
		/// <summary>
		/// Set the move of the server.
		/// </summary>
		/// <param name="msg"> The message to the client</param>
		public static void SetMsg2Client(string msg)
		{
			lock (typeof(EmilChessService))
			{
				if (Msg2Client == null)
					Msg2Client = msg;
				else
					Msg2Client = Msg2Client + "\n>" + msg;
			}
		}
		/// <summary>
		/// Determine if there are a message to the server.
		/// </summary>
		/// <returns> Returns true if there are any messages to the server</returns>
		public static bool HasMsg4Server()
		{
			return Msg2Server != null;
		}
		/// <summary>
		/// Get the messages to the server.
		/// </summary>
		/// <returns> Returns the message to the server</returns>
		public static string GetMsg4Server()
		{
			lock (typeof(EmilChessService))
			{
				string temp=Msg2Server;
				Msg2Server = null;
				return temp;
			}
		}
		/// <summary>
		/// Set the move of the server.
		/// </summary>
		/// <param name="msg"> The message to the server</param>
		public void SetMsg2Server(string msg)
		{
			lock (typeof(EmilChessService))
			{
				if (Msg2Server == null)
					Msg2Server = msg;
				else
					Msg2Server = Msg2Server + "\n>" + msg;
			}
		}
	}
}
